import pyfastaq
from ariba import report, summary_cluster

class Error (Exception): pass

class SummarySample:
    def __init__(self, report_tsv, min_pc_id=90, only_clusters=None):
        self.report_tsv = report_tsv
        self.min_pc_id = min_pc_id
        self.only_clusters = only_clusters
        self.clusters = {}


    def __eq__(self, other):
       return type(other) is type(self) and self.__dict__ == other.__dict__


    @staticmethod
    def _load_file(filename, min_pc_id, only_clusters=None):
        f = pyfastaq.utils.open_file_read(filename)
        clusters = {}

        for line in f:
            if line.startswith('#'):
                if line.rstrip()[1:].split('\t') != report.columns:
                    pyfastaq.utils.close(f)
                    raise Error('Error parsing the following line.\n' + line)
                continue

            data_dict = summary_cluster.SummaryCluster.line2dict(line, filename=filename)
            cluster = data_dict['cluster']
            if only_clusters is not None and cluster not in only_clusters:
                continue

            if cluster not in clusters:
                clusters[cluster] = summary_cluster.SummaryCluster(min_pc_id=min_pc_id)
            clusters[cluster].add_data_dict(data_dict)

        pyfastaq.utils.close(f)

        to_delete = set()

        for cluster_name, cluster in clusters.items():
            cluster.gather_data()
            if cluster.name is None:
                to_delete.add(cluster_name)

        for name in to_delete:
            del clusters[name]

        return clusters


    def _column_summary_data(self):
        return {c: self.clusters[c].column_summary_data() for c in self.clusters}


    def _var_groups(self):
        return {c: self.clusters[c].has_var_groups() for c in self.clusters}


    def _variant_column_names_tuples_and_het_snps(self):
        variants = {}
        het_snps = {}
        for cluster_name, cluster in self.clusters.items():
            cluster_vars = cluster.non_synon_variants()
            cluster_noncoding_het_snps = cluster.known_noncoding_het_snps()

            if len(cluster_vars):
                variants[cluster_name] = cluster_vars
                het_snps[cluster_name] = cluster_noncoding_het_snps
        return variants, het_snps


    def run(self):
        self.clusters = self._load_file(self.report_tsv, self.min_pc_id, only_clusters=self.only_clusters)
        self.column_summary_data = self._column_summary_data()
        self.variant_column_names_tuples, self.het_snps = self._variant_column_names_tuples_and_het_snps()
        self.var_groups = self._var_groups()




import unittest
from ariba import external_progs

class TestExternalProgs(unittest.TestCase):
    def test_external_progs_ok(self):
        '''Test that external programs are found'''
        external_progs.ExternalProgs(verbose=True)




__all__ = [
    'aln2meta',
    'expandflag',
    'flag',
    'getref',
    'micplot',
    'prepareref',
    'prepareref_tb',
    'pubmlstget',
    'pubmlstspecies',
    'refquery',
    'reportfilter',
    'run',
    'summary',
    'test',
    'version',
]

from ariba.tasks import *



import sys
import csv
import os

class Error (Exception): pass

class MlstProfile:
    def __init__(self, infile, duplicate_warnings=True):
        self.infile = infile
        self.duplicate_warnings = duplicate_warnings
        self.columns_to_ignore = ['clonal_complex', 'CC', 'Lineage', 'mlst_clade', 'species']

        if not os.path.exists(self.infile):
            raise Error('Error! Input file "' + self.infile + '" not found.')
        self._load_input_file()


    def _load_input_file(self):
        self.profile_to_type = {}

        with open(self.infile) as f:
            reader = csv.DictReader(f, delimiter='\t')
            if reader.fieldnames[0] != 'ST':
                raise Error('Error. Expected first column of profile file "' + self.infile + '" to be "ST"')

            self.genes_list = [column_name for column_name in reader.fieldnames[1:] if column_name not in self.columns_to_ignore]

            for row in reader:
                type_tuple = tuple(int(row[x]) for x in self.genes_list)
                if type_tuple in self.profile_to_type:
                    previous_st = self.profile_to_type[type_tuple]
                    this_st = int(row['ST'])
                    if previous_st != this_st:
                        new_st = min(previous_st, this_st)
                        if self.duplicate_warnings:
                            print('WARNING: Same profile found twice in input file, but two different STs. Going to use the ST with the smaller number (', new_st, ')', sep='', file=sys.stderr)
                            print(' ... STs are', previous_st, this_st, 'and alleles are', ', '.join([x + ':' + row[x] for x in self.genes_list]), file=sys.stderr)
                        self.profile_to_type[type_tuple] = new_st
                else:
                    self.profile_to_type[type_tuple] = int(row['ST'])

        self.genes_set = set(self.genes_list)


    def has_gene(self, gene):
        return gene in self.genes_set


    def get_sequence_type(self, type_dict):
        key = tuple(type_dict.get(x, 'ND') for x in self.genes_list)
        if 'ND' in key:
            return 'ND'
        else:
            return self.profile_to_type.get(key, 'Novel')





import unittest
import os
import pyfastaq
from ariba import sequence_metadata, sequence_variant

modules_dir = os.path.dirname(os.path.abspath(sequence_metadata.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestSequenceMetadata(unittest.TestCase):
    def test_init_fails_on_bad_lines(self):
        '''Test init fails on bad lines'''
        lines = [
            'only one column. There can NOT be only one\n',
            'two\tcolumns is not enough\n',
            'three\tcolumns\tis still not enough\n',
            'four\tcolumns\tis\talso not enough\n',
            'five\tcolumns\tis\talso\tnot enough\n',
            'seven\tcolumns\tis\tone\tmore\tthan\nwe want',
        ]

        for line in lines:
            with self.assertRaises(sequence_metadata.Error):
                sequence_metadata.SequenceMetadata(line)

        tests = [
            ('gene\tx\t0\t.\t.\tfoo\n', sequence_metadata.Error),
            ('gene\t1\t2\t.\t.\tfoo\n', sequence_metadata.Error),
            ('gene\t1\t1\tI42\t.\tfoo\n', sequence_variant.Error),
        ]

        for line, err in tests:
            with self.assertRaises(err):
                sequence_metadata.SequenceMetadata(line)


    def test_init_on_good_input(self):
        '''test init ok on good input'''
        data = sequence_metadata.SequenceMetadata('gene\t1\t0\tI42L\tid\tspam spam wonderful spam')
        self.assertEqual(data.name, 'gene')
        self.assertEqual(data.seq_type, 'p')
        self.assertEqual(data.variant_only, False)
        self.assertEqual(data.variant.wild_value, 'I')
        self.assertEqual(data.variant.variant_value, 'L')
        self.assertEqual(data.variant.identifier, 'id')
        self.assertEqual(data.free_text, 'spam spam wonderful spam')


    def test_str(self):
        '''test __str__'''
        lines = [
            'gene1\t1\t1\tA42G\tid1\tspam',
            'gene2\t0\t0\t.\t.\t.',
            'gene3\t0\t0\t.\t.\teggs',
            'gene4\t1\t0\tI42K\tid\tthis mutation kills tardigrades',
        ]

        for line in lines:
            self.assertEqual(line, str(sequence_metadata.SequenceMetadata(line)))


    def test_has_variant(self):
        '''test has_variant'''
        tests = [
            ('gene1\t0\t0\t.\t.\t.', False),
            ('gene1\t0\t0\tA2T\t.\t,', True),
            ('gene1\t0\t0\tT2A\t.\t.', False),
            ('gene1\t1\t0\tI2Y\t.\t.', True),
            ('gene1\t1\t0\tY2I\t.\t.', False),
        ]

        seq = pyfastaq.sequences.Fasta('name', 'ATGTATTGCTGA') # translation: MYC*

        for line, expected in tests:
            metadata = sequence_metadata.SequenceMetadata(line)
            self.assertEqual(expected, metadata.has_variant(seq))


    def test_to_string(self):
        '''test to_string'''
        lines = [
            ('gene1', '0', '0', 'A42G', 'id1', 'spam'),
            ('gene2', '0', '0', '.', '.', '.'),
            ('gene3', '0', '0', '.', '.', 'eggs'),
            ('gene4', '1', '0', 'I42K', 'id', 'this mutation kills tardigrades'),
        ]

        for line in lines:
            m = sequence_metadata.SequenceMetadata('\t'.join(line))
            for separator in ('_', '\t'):
                expected = separator.join(line)
                self.assertEqual(expected, m.to_string(separator=separator))




import unittest
import os
import pyfastaq
import pymummer
from ariba import assembly_compare

modules_dir = os.path.dirname(os.path.abspath(assembly_compare.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestAssemblyCompare(unittest.TestCase):
    def test_parse_nucmer_coords_file(self):
        '''test _parse_nucmer_coords_file'''
        coords_file = os.path.join(data_dir, 'assembly_compare_parse_nucmer_coords_file.coords')
        ref_name = 'ref'
        got = assembly_compare.AssemblyCompare._parse_nucmer_coords_file(coords_file, ref_name)
        line1 = ['1', '1000', '1', '1000', '1000', '1000', '100.00', '1000', '1000', '1', '1', 'ref', 'contig1']
        line2 = ['1', '240', '1', '240', '240', '240', '100.00', '1000', '580', '1', '1', 'ref', 'contig2']
        line3 = ['661', '1000', '241', '580', '340', '340', '100.00', '1000', '580', '1', '1', 'ref', 'contig2']
        expected = {
            'contig1': [pymummer.alignment.Alignment('\t'.join(line1))],
            'contig2': [pymummer.alignment.Alignment('\t'.join(line2)), pymummer.alignment.Alignment('\t'.join(line3))],
        }
        self.assertEqual(expected, got)


    def test_nucmer_hits_to_percent_identity(self):
        '''test _nucmer_hits_to_percent_identity'''
        hits = [
            ['1', '10', '1', '10', '10', '10', '90.00', '1000', '1000', '1', '1', 'ref', 'scaff1'],
            ['9', '42', '9', '42', '34', '34', '100.00', '1000', '1000', '1', '1', 'ref', 'scaff1'],
            ['1', '42', '1', '42', '42', '42', '42.42', '1000', '1000', '1', '1', 'ref', 'scaff2'],
        ]
        nucmer_hits = {
            'scaff1': [
                pymummer.alignment.Alignment('\t'.join(hits[0])),
                pymummer.alignment.Alignment('\t'.join(hits[1])),
            ],
            'scaff2': [
                pymummer.alignment.Alignment('\t'.join(hits[2])),
            ]
        }
        expected = {'scaff1': 100.0, 'scaff2': 42.42}
        got = assembly_compare.AssemblyCompare._nucmer_hits_to_percent_identity(nucmer_hits)
        self.assertEqual(expected, got)


    def test_nucmer_hits_to_assembly_coords(self):
        '''test _nucmer_hits_to_assembly_coords'''
        hits = [
            ['1', '10', '1', '10', '10', '10', '100.00', '1000', '1000', '1', '1', 'ref', 'scaff1'],
            ['9', '42', '9', '42', '34', '34', '100.00', '1000', '1000', '1', '1', 'ref', 'scaff1'],
            ['50', '52', '50', '52', '3', '3', '100.00', '1000', '1000', '1', '1', 'ref', 'scaff1'],
            ['1', '42', '1', '42', '42', '42', '100.00', '1000', '1000', '1', '1', 'ref', 'scaff2'],
        ]
        nucmer_hits = {
            'scaff1': [
                pymummer.alignment.Alignment('\t'.join(hits[0])),
                pymummer.alignment.Alignment('\t'.join(hits[1])),
                pymummer.alignment.Alignment('\t'.join(hits[2])),
            ],
            'scaff2': [
                pymummer.alignment.Alignment('\t'.join(hits[3])),
            ]
        }
        expected = {
            'scaff1': [
                pyfastaq.intervals.Interval(0, 41),
                pyfastaq.intervals.Interval(49, 51)
            ],
            'scaff2': [
                pyfastaq.intervals.Interval(0, 41),
            ]
        }
        got = assembly_compare.AssemblyCompare._nucmer_hits_to_assembly_coords(nucmer_hits)
        self.assertEqual(expected, got)


    def test_nucmer_hits_to_ref_coords(self):
        '''test nucmer_hits_to_ref_coords'''
        hits = [
            ['1', '42', '1', '42', '42', '42', '100.00', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['31', '52', '1', '22', '22', '22', '100.00', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['100', '142', '200', '242', '42', '42', '99.42', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['100', '110', '200', '210', '11', '11', '99.42', '1000', '1000', '1', '1', 'ref', 'contig2'],
        ]
        nucmer_hits = {
            'contig1': [
                pymummer.alignment.Alignment('\t'.join(hits[0])),
                pymummer.alignment.Alignment('\t'.join(hits[1])),
                pymummer.alignment.Alignment('\t'.join(hits[2])),
            ],
            'contig2': [
                pymummer.alignment.Alignment('\t'.join(hits[3])),
            ]
        }
        got = assembly_compare.AssemblyCompare.nucmer_hits_to_ref_coords(nucmer_hits)
        expected = {
            'contig1': [pyfastaq.intervals.Interval(0,51), pyfastaq.intervals.Interval(99, 141)],
            'contig2': [pyfastaq.intervals.Interval(99, 109)]
        }

        self.assertEqual(expected, got)

        got = assembly_compare.AssemblyCompare.nucmer_hits_to_ref_coords(nucmer_hits, contig='contig2')
        del expected['contig1']
        self.assertEqual(expected, got)


    def test_nucmer_hits_to_ref_and_qry_coords(self):
        '''test _nucmer_hits_to_ref_and_qry_coords'''
        hits = [
            ['1', '42', '1', '42', '42', '42', '100.00', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['31', '52', '1', '22', '22', '22', '100.00', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['11', '32', '1000', '1022', '22', '22', '100.00', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['100', '142', '200', '242', '42', '42', '99.42', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['100', '110', '200', '210', '11', '11', '99.42', '1000', '1000', '1', '1', 'ref', 'contig2'],
        ]
        nucmer_hits = {
            'contig1': [
                pymummer.alignment.Alignment('\t'.join(hits[0])),
                pymummer.alignment.Alignment('\t'.join(hits[1])),
                pymummer.alignment.Alignment('\t'.join(hits[2])),
            ],
            'contig2': [
                pymummer.alignment.Alignment('\t'.join(hits[3])),
            ]
        }

        got_ctg, got_ref = assembly_compare.AssemblyCompare.nucmer_hits_to_ref_and_qry_coords(nucmer_hits)
        expected_ctg = {
            'contig1': [pyfastaq.intervals.Interval(0,51), pyfastaq.intervals.Interval(99, 141)],
            'contig2': [pyfastaq.intervals.Interval(99, 109)]
        }
        expected_ref = {
            'contig1': [pyfastaq.intervals.Interval(0,51), pyfastaq.intervals.Interval(99, 141)],
            'contig2': [pyfastaq.intervals.Interval(99, 109)]
        }


    def test_ref_cov_per_contig(self):
        '''test ref_cov_per_contig'''
        hits = [
            ['1', '42', '1', '42', '42', '42', '100.00', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['100', '142', '200', '242', '42', '42', '99.42', '1000', '1000', '1', '1', 'ref', 'contig1'],
            ['100', '110', '200', '210', '11', '11', '99.42', '1000', '1000', '1', '1', 'ref', 'contig2'],
        ]
        nucmer_hits = {
            'contig1': [
                pymummer.alignment.Alignment('\t'.join(hits[0])),
                pymummer.alignment.Alignment('\t'.join(hits[1])),
            ],
            'contig2': [
                pymummer.alignment.Alignment('\t'.join(hits[2])),
            ]
        }

        expected = {'contig1': 85, 'contig2': 11}
        got = assembly_compare.AssemblyCompare.ref_cov_per_contig(nucmer_hits)
        self.assertEqual(expected, got)


    def test_get_assembled_reference_sequences(self):
        '''test _get_assembled_reference_sequences'''
        ref_sequence = pyfastaq.sequences.Fasta('ref_seq', 'ATGGTACAAGACGGCCCTTTGCAGTCCTGTGTACTTGCGGGTCGCTCCTTTGCATTGAATTATCGAACATCGTCGCGTTCAAGATCCCGCGAAAAAAATTATAGATCGCAGGATATCACTGCCAGTGGCATCTGTGTAAGCGCTTAG')
        assembly = {
            'contig1': pyfastaq.sequences.Fasta('contig1', 'CATCTATGCTGCATCGATCACTGACGTATCATCATCAGCGTACTGACGTATTAGTTTGTAATGGTACAAGACGGCCCTTTGCAGTCCTGTGTACTTGCGGGTCGCTCCTTTGCATTGAATTATCGAACATCGTCGCGTTCAAGATCCCGCGAAAAAAATTATAGATCGCAGGATATCACTGCCAGTGGCATCTGTGTAAGCGCTTAGACGTCGTACTACTGTATATGCATCGATCTGAA'),
            'contig2': pyfastaq.sequences.Fasta('contig2', 'AGTGATATCCTGCGATCTATAATTTTTTTCGCGGGATCTTGAACGCGACGATGTTCGATAATTCAATGCAAAGGAGCGACCCGCAAGTACACAGGACTGCAAA')
        }

        hits = [
            ['1', '147', '61', '207', '147', '147', '100.00', '147', '239', '1', '1', 'ref_seq', 'contig1'],
            ['18', '120', '103', '1', '103', '103', '100.00', '147', '103', '1', '-1', 'ref_seq', 'contig2']
        ]
        nucmer_hits = {
            'contig1': [
                pymummer.alignment.Alignment('\t'.join(hits[0])),
            ],
            'contig2': [
                pymummer.alignment.Alignment('\t'.join(hits[1])),
            ]
        }

        expected = {'ref_seq.1.147.contig1.61.207.+.complete': pyfastaq.sequences.Fasta('ref_seq.1.147.contig1.61.207.+.complete', 'ATGGTACAAGACGGCCCTTTGCAGTCCTGTGTACTTGCGGGTCGCTCCTTTGCATTGAATTATCGAACATCGTCGCGTTCAAGATCCCGCGAAAAAAATTATAGATCGCAGGATATCACTGCCAGTGGCATCTGTGTAAGCGCTTAG'),
            'ref_seq.18.120.contig2.1.103.-': pyfastaq.sequences.Fasta('ref_seq.18.120.contig2.1.103.-', 'TTTGCAGTCCTGTGTACTTGCGGGTCGCTCCTTTGCATTGAATTATCGAACATCGTCGCGTTCAAGATCCCGCGAAAAAAATTATAGATCGCAGGATATCACT')
        }

        got = assembly_compare.AssemblyCompare._get_assembled_reference_sequences(nucmer_hits, ref_sequence, assembly)
        self.assertEqual(expected, got)


    def test_whole_gene_covered_by_nucmer_hits(self):
        '''test _whole_gene_covered_by_nucmer_hits'''
        ref_seq = pyfastaq.sequences.Fasta('ref', 'ACGTGTGCAT')
        hit1 = ['1', '10', '1', '10', '10', '10', '100.00', '10', '10', '1', '1', 'ref', 'contig1']
        hit2 = ['1', '5', '1', '5', '5', '5', '100.00', '10', '10', '1', '1', 'ref', 'contig2']
        hit3 = ['6', '10', '6', '10', '5', '5', '100.00', '10', '10', '1', '1', 'ref', 'contig2']
        nucmer_hits = [
            {'contig1': [pymummer.alignment.Alignment('\t'.join(hit1))]},
            {'contig2': [pymummer.alignment.Alignment('\t'.join(hit2))]},
            {'contig2': [pymummer.alignment.Alignment('\t'.join(hit2)), pymummer.alignment.Alignment('\t'.join(hit3))]}
        ]
        expected = [True, False, True]
        for i in range(len(nucmer_hits)):
            got = assembly_compare.AssemblyCompare._whole_gene_covered_by_nucmer_hits(nucmer_hits[i], ref_seq, 0.95, 0)
            self.assertEqual(expected[i], got)


    def test_ref_has_region_assembled_twice(self):
        '''test _ref_has_region_assembled_twice'''
        ref_seq = pyfastaq.sequences.Fasta('gene', 'ACGTGTGCAT')
        hit1 = ['1', '10', '1', '10', '10', '10', '100.00', '10', '10', '1', '1', 'gene', 'contig1']
        hit2 = ['1', '5', '1', '5', '5', '5', '100.00', '10', '10', '1', '1', 'gene', 'contig2']
        nucmer_hits = { 'contig1': [pymummer.alignment.Alignment('\t'.join(hit1))] }
        self.assertFalse(assembly_compare.AssemblyCompare._ref_has_region_assembled_twice(nucmer_hits, ref_seq, 0.03))
        nucmer_hits['contig2'] = [pymummer.alignment.Alignment('\t'.join(hit2))]
        self.assertTrue(assembly_compare.AssemblyCompare._ref_has_region_assembled_twice(nucmer_hits, ref_seq, 0.03))


    def test_longest_nucmer_hit_in_ref(self):
        '''test _longest_nucmer_hit_in_ref'''
        hits = [
            ['1', '39', '1', '39', '39', '39', '100.00', '39', '39', '1', '1', 'gene', 'contig1'],
            ['1', '20', '1', '20', '20', '20', '100.00', '39', '39', '1', '1', 'gene', 'contig1'],
            ['21', '39', '21', '39', '19', '19', '100.00', '39', '39', '1', '1', 'gene', 'contig2'],
        ]
        alignments = [pymummer.alignment.Alignment('\t'.join(x)) for x in hits]
        nucmer_hits = {
            'contig1': [alignments[0]],
            'contig2': [alignments[1], alignments[2]],
        }
        got = assembly_compare.AssemblyCompare._longest_nucmer_hit_in_ref(nucmer_hits)
        self.assertEqual(alignments[0], got)


    def test_find_previous_start_codon(self):
        '''test _find_previous_start_codon'''
        tests = [
            ('ATGTTTAAA', 0, 0, 0),
            ('TATGTTTAAA', 0, 0, None),
            ('TATGTTTAAA', 1, 0, 1),
            ('ATATGTTTAAA', 2, 0, 2),
            ('AATGTTTAAA', 7, 6, None),
            ('AATGTTTAAA', 7, 5, None),
            ('AATGTTTAAA', 7, 4, None),
            ('AATGTTTAAA', 7, 3, None),
            ('AATGTTTAAA', 7, 2, None),
            ('AATGTTTAAA', 7, 1, 1),
            ('AATGTTTAAA', 7, 0, 1),
            ('AGTGTTTAAA', 7, 0, None),
            ('AATGTAGAAA', 7, 0, None),
        ]

        for seq, start_coord, min_coord, expected in tests:
            fa = pyfastaq.sequences.Fasta('x', seq)
            got = assembly_compare.AssemblyCompare._find_previous_start_codon(fa, start_coord, min_coord)
            self.assertEqual(expected, got)


    def test_find_next_stop_codon(self):
        '''test _find_next_stop_codon'''
        tests = [
            ('ATGTTTAGA', 2, 5, None),
            ('ATGTTTAGA', 2, 6, None),
            ('ATGTTTAGA', 2, 7, 5),
            ('ATGTTTGGA', 2, 4, None),
            ('ATGTTTAGA', 5, 6, None),
            ('ATGTTTAGA', 5, 7, 5),
            ('ATGTTTGGA', 5, 7, None),
            ('ATGTTTAGA', 4, 7, None),
            ('ATGTTTAGA', 3, 7, None),
        ]

        for seq, end_coord, max_coord, expected in tests:
            fa = pyfastaq.sequences.Fasta('x', seq)
            got = assembly_compare.AssemblyCompare._find_next_stop_codon(fa, end_coord, max_coord)
            self.assertEqual(expected, got)


    def test_gene_from_nucmer_match(self):
        '''test _gene_from_nucmer_match'''
        tests = [
            (
             ['1', '15', '2', '16', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGTAGTTTCCCTAGATAT', 5,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGTAGTTTCCCTAG'), 'HAS_STOP', None, None)
            ),
            (
             ['1', '15', '2', '16', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 5,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 0, 0)
            ),
            (
             ['1', '15', '2', '15', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 5,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 0, 1)
            ),
            (
             ['1', '15', '2', '14', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 5,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 0, 2)
            ),
            (
             ['1', '15', '2', '13', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 5,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 0, 3)
            ),
            (
             ['2', '15', '3', '16', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 5,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 1, 0)
            ),
            (
             ['3', '15', '4', '16', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 5,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 2, 0)
            ),
            (
             ['4', '15', '5', '16', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 5,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 3, 0)
            ),
            (
             ['1', '15', '2', '14', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 0,
             (pyfastaq.sequences.Fasta('contig.2-13', 'ATGAAATTTCCC'), 'START_OR_END_FAIL', 0, None)
            ),
            (
             ['1', '15', '2', '14', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 1,
             (pyfastaq.sequences.Fasta('contig.2-13', 'ATGAAATTTCCC'), 'START_OR_END_FAIL', 0, None)
            ),
            (
             ['1', '15', '2', '14', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 2,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 0, 2)
            ),
            (
             ['2', '15', '3', '14', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'AATGAAATTTCCCTAGATAT', 2,
             (pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 1, 2)
            ),
            (
             ['2', '15', '18', '7', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig'], 'ATATCTAGGGAAATTTCATT', 2,
             (pyfastaq.sequences.Fasta('contig.16-2', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 1, 2)
            ),
        ]

        for hit, seq, max_extend, expected in tests:
            nucmer_match = pymummer.alignment.Alignment('\t'.join(hit))
            contig = pyfastaq.sequences.Fasta('contig', seq)
            got = assembly_compare.AssemblyCompare._gene_from_nucmer_match(nucmer_match, contig, max_extend)
            self.assertEqual(expected, got)


    def test_get_gene_matching_ref(self):
        '''test _get_gene_matching_ref'''
        hit1 = ['2', '15', '3', '14', '11', '11', '100.00', '20', '20', '1', '1', 'ref', 'contig']
        hit2 = ['2', '7', '3', '8', '6', '6', '100.00', '20', '20', '1', '1', 'ref', 'contig2']
        contigs = {
            'contig': pyfastaq.sequences.Fasta('contig', 'AATGAAATTTCCCTAGATAT'),
            'contig2': pyfastaq.sequences.Fasta('contig2', 'AATGAAATTTCCCTAGATAT')
        }
        nucmer_hits = {
            'contig': [pymummer.alignment.Alignment('\t'.join(hit1))],
            'contig2': [pymummer.alignment.Alignment('\t'.join(hit2))],
        }

        got = assembly_compare.AssemblyCompare._get_gene_matching_ref(nucmer_hits, contigs, 10)
        expected = ('contig', pyfastaq.sequences.Fasta('contig.2-16', 'ATGAAATTTCCCTAG'), 'GENE_FOUND', 1, 2)
        self.assertEqual(expected, got)


    def test_ref_covered_by_at_least_one_full_length_contig(self):
        '''test _ref_covered_by_at_least_one_full_length_contig'''
        hits = [
            ['1', '100', '1', '100', '100', '100', '100.00', '100', '100', '1', '1', 'ref', 'contig1'],
            ['1', '99', '1', '99', '99', '99', '100.00', '100', '100', '1', '1', 'ref', 'contig1'],
            ['1', '96', '1', '96', '96', '96', '100.00', '100', '100', '1', '1', 'ref', 'contig1'],
            ['1', '95', '1', '95', '95', '95', '100.00', '100', '100', '1', '1', 'ref', 'contig1'],
            ['1', '94', '1', '94', '94', '94', '100.00', '100', '100', '1', '1', 'ref', 'contig1'],
        ]
        nucmer_hits = [{'contig1': [pymummer.alignment.Alignment('\t'.join(hit))]} for hit in hits]
        expected = [True, True, True, True, False]
        assert len(expected) == len(nucmer_hits)
        for i in range(len(expected)):
            self.assertEqual(expected[i], assembly_compare.AssemblyCompare._ref_covered_by_at_least_one_full_length_contig(nucmer_hits[i], 0.95, 0))


    def test_nucmer_hit_containing_reference_position(self):
        '''test nucmer_hit_containing_reference_position'''
        listhit1 = ['100', '200', '300', '400', '100', '100', '100.00', '600', '500', '1', '1', 'ref', 'contig1']
        listhit2 = ['400', '500', '500', '600', '100', '100', '100.00', '600', '600', '1', '1', 'ref', 'contig2']
        hit1 = pymummer.alignment.Alignment('\t'.join(listhit1))
        hit2 = pymummer.alignment.Alignment('\t'.join(listhit2))
        nucmer_hits = {
            'contig1': [hit1],
            'contig2': [hit2],
        }

        tests = [
            ('ref2', 150, None),
            ('ref', 42, None),
            ('ref', 98, None),
            ('ref', 200, None),
            ('ref', 99, hit1),
            ('ref', 142, hit1),
            ('ref', 199, hit1),
            ('ref', 200, None),
            ('ref', 398, None),
            ('ref', 399, hit2),
            ('ref', 442, hit2),
            ('ref', 499, hit2),
            ('ref', 500, None),
        ]

        for ref_name, ref_pos, expected in tests:
            got = assembly_compare.AssemblyCompare.nucmer_hit_containing_reference_position(nucmer_hits, ref_name, ref_pos)
            self.assertEqual(expected, got)



import unittest
import os
import copy
import filecmp
import pyfastaq
from ariba import common, mlst_profile, pubmlst_ref_preparer

modules_dir = os.path.dirname(os.path.abspath(pubmlst_ref_preparer.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestPubmlstRefPreparer(unittest.TestCase):
    def test_filter_seq_dict(self):
        '''test _filter_seq_dict'''
        d = {
            'seq1': pyfastaq.sequences.Fasta('seq1', 'A'),
            'seq2': pyfastaq.sequences.Fasta('seq2', 'ACGTA'),
            'seq3': pyfastaq.sequences.Fasta('seq3', 'ACGTT'),
            'seq4': pyfastaq.sequences.Fasta('seq4', 'AACGC'),
            'seq5': pyfastaq.sequences.Fasta('seq5', 'AACGTGTGGTTGGT'),
        }

        expect = copy.copy(d)
        del expect['seq1']
        del expect['seq5']
        pubmlst_ref_preparer.PubmlstRefPreparer._filter_seq_dict(d)
        self.assertEqual(expect, d)


    def test_load_fasta_files_and_write_clusters_file(self):
        '''test _load_fasta_files_and_write_clusters_file'''
        indir = os.path.join(data_dir, 'pubmlst_ref_prepare.test_load_fa_and_clusters.in')
        outdir = 'tmp.test.pubmlst_ref_prepare.test_load_fa_and_clusters'
        os.mkdir(outdir)
        r_prep = pubmlst_ref_preparer.PubmlstRefPreparer('species', outdir)
        profile_file = os.path.join(indir, 'profile.txt')
        r_prep.profile = mlst_profile.MlstProfile(profile_file)
        r_prep._load_fasta_files_and_write_clusters_file(indir)
        expected_cluster_tsv = os.path.join(data_dir, 'pubmlst_ref_prepare.test_load_fa_and_clusters.expect.tsv')
        self.assertTrue(filecmp.cmp(expected_cluster_tsv, r_prep.clusters_file, shallow=False))
        common.rmtree(outdir)

        expected_fasta_files = [os.path.join(indir, x) for x in ['gene1.tfa', 'gene2.tfa']]
        self.assertEqual(expected_fasta_files, r_prep.fasta_files)

        expected_seqs = {
            'gene1': {
                'gene1_1': pyfastaq.sequences.Fasta('gene1_1', 'ACGT'),
                'gene1_2': pyfastaq.sequences.Fasta('gene1_2', 'AAAA'),
            },
            'gene2': {
                'gene2_1': pyfastaq.sequences.Fasta('gene2_1', 'GGGG'),
                'gene2_2': pyfastaq.sequences.Fasta('gene2_2', 'TTTT'),
            },
        }
        self.assertEqual(expected_seqs, r_prep.sequences)




import unittest
import os
import pyfastaq
from ariba import sequence_variant

modules_dir = os.path.dirname(os.path.abspath(sequence_variant.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestSequenceVariant(unittest.TestCase):
    def test_init_fails_on_bad_variant_strings(self):
        '''Test init fails on bad variant strings'''
        bad_variants = [
            'x',
            'x1',
            '1x',
            '1x1',
            'I42K43',
            'I-1K',
        ]

        for var in bad_variants:
            with self.assertRaises(sequence_variant.Error):
                sequence_variant.Variant('p', var, '.')


    def test_init_ok(self):
        '''Test init ok'''
        variants = [('I42K', '.'), ('i42k', 'id1'), ('I42k', 'id2'), ('i42K', 'id3')]

        for var, identifier in variants:
            aa_var = sequence_variant.Variant('p', var, identifier)
            self.assertEqual(41, aa_var.position)
            self.assertEqual('I', aa_var.wild_value)
            self.assertEqual('K', aa_var.variant_value)
            if identifier == '.':
                self.assertIsNone(aa_var.identifier)
            else:
                self.assertEqual(identifier, aa_var.identifier)


    def test_init_str(self):
        '''Test init ok and str'''
        variants = ['I42K', 'i42k', 'I42k', 'i42K']
        expected = 'I42K'

        for var in variants:
            self.assertEqual(expected, str(sequence_variant.Variant('p', var, '.')))


    def test_sanity_check_against_seq_no_translate(self):
        '''test sanity_check_against_seq with translate False'''
        seq = 'BrissSpecialStvff'
        tests = [
            ('I3K', True),
            ('K3I', True),
            ('A2b', False),
            ('x1000y', False)
        ]

        for var, expected in tests:
            variant = sequence_variant.Variant('p', var, '.')
            self.assertEqual(expected, variant.sanity_check_against_seq(seq))


    def test_sanity_check_against_seq_translate(self):
        '''test sanity_check_against_seq with translate True'''
        seq = 'AGTACGACGTAC'  # translates to STTY
        tests = [
            ('S1X', True),
            ('x1s', True),
            ('a1y', False),
            ('x5y', False)
        ]

        for var, expected in tests:
            variant = sequence_variant.Variant('p', var, '.')
            self.assertEqual(expected, variant.sanity_check_against_seq(seq, translate_seq=True))


    def test_has_variant(self):
        '''test has_variant'''
        seq = pyfastaq.sequences.Fasta('name', 'ATGTATTGCTGA') # translation: MYC*
        tests = [
            (sequence_variant.Variant('n', 'A2T', '.'), True),
            (sequence_variant.Variant('n', 'T2A', '.'), False),
            (sequence_variant.Variant('p', 'I2Y', '.'), True),
            (sequence_variant.Variant('p', 'Y2I', '.'), False),
        ]

        for var, expected in tests:
            self.assertEqual(expected, var.has_variant(seq))


    def test_nucleotide_range(self):
        '''test nucleotide_range'''
        sv = sequence_variant.Variant('n', 'A2T', '.')
        self.assertEqual((1, 1), sv.nucleotide_range())
        sv = sequence_variant.Variant('p', 'I42L', '.')
        self.assertEqual((123, 125), sv.nucleotide_range())



import os
import tempfile

from ariba import common, external_progs

class Error (Exception): pass

class ReadFilter:
    def __init__(self,
        readstore_obj,
        references_fa,
        cluster_name,
        log_fh,
        extern_progs=None,
    ):
        self.readstore = readstore_obj
        self.references_fa = references_fa
        self.cluster_name = cluster_name
        self.log_fh = log_fh

        if extern_progs is None:
            self.extern_progs = external_progs.ExternalProgs(using_spades=False)
        else:
            self.extern_progs = extern_progs


    @staticmethod
    def _run_cdhit_est_2d(reference, reads, outfile, cdhitest2d, verbose=False, verbose_fh=None):
        cmd = ' '.join([
            cdhitest2d,
            '-i', reference,
            '-i2', reads,
            '-G 0 -M 0 -d 0 -aS 0.95',
            '-o', outfile
        ])
        common.syscall(cmd, verbose=verbose, verbose_filehandle=verbose_fh)
        os.unlink(outfile)


    @staticmethod
    def _cdhit_clstr_to_reads(infile):
        reads = set()

        with open(infile) as f:
            for line in f:
                if line.startswith('>') or line.endswith('*\n'):
                    continue

                fields = line.split()
                if not (fields[2].startswith('>') and fields[2].endswith('...')):
                    raise Error('Error parsing the following line of cd-hit-est-2d output from file "' + infile + '":\n' + line)

                reads.add(int(fields[2][1:-5]))

        return reads


    def run(self, reads_out1, reads_out2):
        tmpdir = tempfile.mkdtemp(prefix='tmp.filter_reads.', dir=os.getcwd())
        all_reads_fasta = os.path.join(tmpdir, 'all_reads_for_cdhit.fa')
        self.readstore.get_reads(self.cluster_name, all_reads_fasta, fasta=True, log_fh=self.log_fh)
        cdhit_out = os.path.join(tmpdir, 'cdhit')
        ReadFilter._run_cdhit_est_2d(
            self.references_fa,
            all_reads_fasta,
            cdhit_out,
            self.extern_progs.exe('cdhit2d'),
            verbose=True,
            verbose_fh=self.log_fh
        )

        wanted_read_ids = ReadFilter._cdhit_clstr_to_reads(cdhit_out + '.clstr')
        total_reads, total_bases = self.readstore.get_reads(
            self.cluster_name,
            reads_out1,
            out2=reads_out2,
            log_fh=self.log_fh,
            wanted_ids=wanted_read_ids
        )

        common.rmtree(tmpdir)
        return total_reads, total_bases



import unittest
import filecmp
import os
from ariba import summary, summary_sample

modules_dir = os.path.dirname(os.path.abspath(summary.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')

class TestSummary(unittest.TestCase):
    def test_init(self):
        '''Test init'''
        fofn = os.path.join(data_dir, 'summary_test_init.fofn')
        s = summary.Summary('out', fofn=fofn)
        self.assertEqual(s.filenames, {'file1': None, 'file2': None})
        s = summary.Summary('out', filenames={'file42': None})
        self.assertEqual(s.filenames, {'file42': None})
        s = summary.Summary('out', fofn=fofn, filenames={'file42': None})
        self.assertEqual(s.filenames, {'file42': None, 'file1': None, 'file2': None})


    def test_determine_cluster_cols(self):
        col_strings = [
            'assembled,match,ref_seq,pct_id,ctg_cov,known_var,novel_var',
            'assembled,match,ref_seq,pct_id,known_var,novel_var',
            'ref_seq,pct_id,known_var,novel_var',
            'assembled,pct_id,known_var,novel_var',
            'assembled',
            '',
            None,
        ]

        expected = [
            {'assembled': True, 'match': True, 'ref_seq': True, 'pct_id': True, 'ctg_cov': True, 'known_var': True, 'novel_var': True},
            {'assembled': True, 'match': True, 'ref_seq': True, 'pct_id': True, 'ctg_cov': False, 'known_var': True, 'novel_var': True},
            {'assembled': False, 'match': False, 'ref_seq': True, 'pct_id': True, 'ctg_cov': False, 'known_var': True, 'novel_var': True},
            {'assembled': True, 'match': False, 'ref_seq': False, 'pct_id': True, 'ctg_cov': False, 'known_var': True, 'novel_var': True},
            {'assembled': True, 'match': False, 'ref_seq': False, 'pct_id': False, 'ctg_cov': False, 'known_var': False, 'novel_var': False},
            {'assembled': False, 'match': False, 'ref_seq': False, 'pct_id': False, 'ctg_cov': False, 'known_var': False, 'novel_var': False},
            {'assembled': False, 'match': False, 'ref_seq': False, 'pct_id': False, 'ctg_cov': False, 'known_var': False, 'novel_var': False},
        ]

        assert len(col_strings) == len(expected)

        for i in range(len(col_strings)):
            self.assertEqual(expected[i], summary.Summary._determine_cluster_cols(col_strings[i]))


    def test_load_fofn(self):
        '''Test _load_fofn'''
        infile = os.path.join(data_dir, 'summary_test_load_fofn')
        got = summary.Summary._load_fofn(infile)
        expected = {
            '/foo/bar/abc.tsv': None,
            '/spam/eggs/file1.tsv': 'short_name1',
            '/spam/eggs/file2.tsv': 'short_name2',
            '/spam/eggs/file3.tsv': None
        }
        self.assertEqual(expected, got)


    def test_load_input_files(self):
        '''Test _load_input_files'''
        file1 = os.path.join(data_dir, 'summary_test_load_input_files.1.tsv')
        file2 = os.path.join(data_dir, 'summary_test_load_input_files.2.tsv')
        sample1 = summary_sample.SummarySample(file1)
        sample2 = summary_sample.SummarySample(file2)
        sample1.run()
        sample2.run()
        got = summary.Summary._load_input_files([file1, file2], 90)
        expected = {file1: sample1, file2: sample2}
        self.assertEqual(expected, got)

        sample1 = summary_sample.SummarySample(file1, only_clusters={'noncoding1'})
        sample2 = summary_sample.SummarySample(file2, only_clusters={'noncoding1'})
        sample1.run()
        sample2.run()
        expected = {file1: sample1, file2: sample2}
        got = summary.Summary._load_input_files([file1, file2], 90, only_clusters={'noncoding1'})
        self.assertEqual(expected, got)


    def test_gather_unfiltered_output_data(self):
        '''test gather_unfiltered_output_data'''
        infiles = [
            os.path.join(data_dir, 'summary_gather_unfiltered_output_data.in.1.tsv'),
            os.path.join(data_dir, 'summary_gather_unfiltered_output_data.in.2.tsv')
        ]

        expected_all = {
            infiles[0]: {
                'noncoding1': {
                    'summary': {
                        'assembled': 'yes',
                        'known_var': 'yes',
                        'match': 'yes',
                        'novel_var': 'no',
                        'pct_id': '98.33',
                        'ctg_cov': '10.0',
                        'ref_seq': 'noncoding_ref1'
                    },
                    'groups': {},
                    'vars': {},
                },
                'noncoding2': {
                    'summary': {
                        'assembled': 'yes',
                        'known_var': 'yes',
                        'match': 'yes',
                        'novel_var': 'no',
                        'pct_id': '98.33',
                        'ctg_cov': '10.0',
                        'ref_seq': 'noncoding_ref2'
                    },
                    'groups': {},
                    'vars': {},
                },
                'presence_absence1': {
                    'summary': {
                        'assembled': 'yes',
                        'known_var': 'no',
                        'match': 'yes',
                        'novel_var': 'yes',
                        'pct_id': '98.96',
                        'ctg_cov': '20.1',
                        'ref_seq': 'presence_absence_ref1'
                    },
                    'groups': {},
                    'vars': {},
                },
                'presence_absence2': {
                    'summary': {
                            'assembled': 'partial',
                            'known_var': 'no',
                            'match': 'no',
                            'novel_var': 'yes',
                            'pct_id': '99.1',
                            'ctg_cov': '22.3',
                            'ref_seq': 'presence_absence_ref2'
                    },
                    'groups': {},
                    'vars': {}
                }
            },
            infiles[1]: {
                'noncoding1': {
                    'summary': {'assembled': 'yes',
                        'known_var': 'yes',
                        'match': 'yes',
                        'novel_var': 'no',
                        'pct_id': '98.33',
                        'ctg_cov': '50.1',
                        'ref_seq': 'noncoding_ref1'
                     },
                    'groups': {},
                    'vars': {},
                },
                'noncoding2': {
                    'summary': {
                        'assembled': 'yes',
                        'known_var': 'yes',
                        'match': 'yes',
                        'novel_var': 'no',
                        'pct_id': '98.33',
                        'ctg_cov': '10.0',
                        'ref_seq': 'noncoding_ref2'
                    },
                    'groups': {},
                    'vars': {},
                },
                'presence_absence1': {
                    'summary': {
                            'assembled': 'yes',
                            'known_var': 'no',
                            'match': 'yes',
                            'novel_var': 'yes',
                            'pct_id': '98.96',
                            'ctg_cov': '51.1',
                            'ref_seq': 'presence_absence1'
                    },
                    'groups': {},
                    'vars': {}
                },
            }
        }

        expected_potential_cols = {
            'noncoding1': {
                'summary': {
                    'assembled',
                    'known_var',
                    'match',
                    'novel_var',
                    'pct_id',
                    'ctg_cov',
                    'ref_seq'
                },
                'groups': set(),
                'vars': set()
            },
            'noncoding2': {
                'summary': {
                    'assembled',
                    'known_var',
                    'match',
                    'novel_var',
                    'pct_id',
                    'ctg_cov',
                    'ref_seq'
                },
                'groups': set(),
                'vars': set()
            },
            'presence_absence1': {
                'summary': {
                    'assembled',
                    'known_var',
                    'match',
                    'novel_var',
                    'pct_id',
                    'ctg_cov',
                    'ref_seq'
                },
                'groups': set(),
                'vars': set()
            },
            'presence_absence2': {
                'summary': {
                    'assembled',
                    'known_var',
                    'match',
                    'novel_var',
                    'pct_id',
                    'ctg_cov',
                    'ref_seq'
                },
                'groups': set(),
                'vars': set()
            }
        }

        self.maxDiff = None
        s = summary.Summary('out', filenames=infiles)
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        self.assertEqual(expected_potential_cols, s.all_potential_columns)
        self.assertEqual(expected_all, s.all_data)

        expected_potential_cols['noncoding1']['groups'] = {'id3', 'id1', 'id1.%', 'id3.%'}
        expected_potential_cols['noncoding2']['groups'] = {'id2.%', 'id2'}
        expected_all[infiles[0]]['noncoding1']['groups'] = {'id1': 'yes', 'id1.%': 100.0}
        expected_all[infiles[0]]['noncoding2']['groups'] = {'id2': 'yes_multi_het', 'id2.%': 'NA'}
        expected_all[infiles[1]]['noncoding1']['groups'] = {'id1': 'het', 'id1.%': 80.0, 'id3': 'yes', 'id3.%': 100.0}
        expected_all[infiles[1]]['noncoding2']['groups'] = {'id2': 'het', 'id2.%': 40.0}
        s = summary.Summary('out', filenames=infiles, show_var_groups=True)
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        self.assertEqual(expected_potential_cols, s.all_potential_columns)
        self.assertEqual(expected_all, s.all_data)

        expected_potential_cols['noncoding1']['vars'] = {'14T', '14T.%', '14GT', '14GT.%', '6G', '6G.%'}
        expected_potential_cols['noncoding2']['vars'] = {'52GT', '52GT.%', '42T', '42T.%'}

        expected_all[infiles[0]]['noncoding1']['vars'] = {'14T': 'yes', '14T.%': 100.0}
        expected_all[infiles[0]]['noncoding2']['vars'] = {'42T': 'yes', '42T.%': 100.0, '52GT': 'het', '52GT.%': 40.0}
        expected_all[infiles[1]]['noncoding1']['vars'] = {'14GT': 'het', '14GT.%': 80.0, '6G': 'yes', '6G.%': 100.0}
        expected_all[infiles[1]]['noncoding2']['vars'] = {'52GT': 'het', '52GT.%': 40.0}
        s = summary.Summary('out', filenames=infiles, show_var_groups=True, show_known_vars=True)
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        self.assertEqual(expected_potential_cols, s.all_potential_columns)
        self.assertEqual(expected_all, s.all_data)

        expected_potential_cols['presence_absence1']['vars'] = {'A10V'}
        expected_potential_cols['presence_absence2']['vars'] = {'V175L'}
        expected_all[infiles[0]]['presence_absence1']['vars'] = {'A10V': 'yes'}
        expected_all[infiles[0]]['presence_absence2']['vars'] = {'V175L': 'yes'}
        expected_all[infiles[1]]['presence_absence1']['vars'] = {'A10V': 'yes'}
        s = summary.Summary('out', filenames=infiles, show_var_groups=True, show_known_vars=True, show_novel_vars=True)
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        self.assertEqual(expected_potential_cols, s.all_potential_columns)
        self.assertEqual(expected_all, s.all_data)


    def test_to_matrix_all_cols(self):
        '''Test _to_matrix all columns'''
        infiles = [
            os.path.join(data_dir, 'summary_to_matrix.1.tsv'),
            os.path.join(data_dir, 'summary_to_matrix.2.tsv')
        ]

        fofn = 'tmp.summary_to_matrix_all_cols'
        with open(fofn, 'w') as f:
            print(infiles[0], 'sample1', file=f)
            print(infiles[1], file=f)


        s = summary.Summary('out', fofn=fofn, show_var_groups=True, show_known_vars=True, show_novel_vars=True)
        os.unlink(fofn)
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        got_phandango_header, got_csv_header, got_matrix = summary.Summary._to_matrix(s.filenames, s.all_data, s.all_potential_columns, s.cluster_columns)

        expected_phandango_header = ['name', 'noncoding1.assembled:o1', 'noncoding1.match:o1', 'noncoding1.ref_seq:o2', 'noncoding1.pct_id:c1', 'noncoding1.ctg_cov:c3', 'noncoding1.known_var:o1', 'noncoding1.novel_var:o1', 'noncoding1.id1:o1', 'noncoding1.id1.%:c2', 'noncoding1.id3:o1', 'noncoding1.id3.%:c2', 'noncoding1.14GT:o1', 'noncoding1.14GT.%:c2', 'noncoding1.14T:o1', 'noncoding1.14T.%:c2', 'noncoding1.6G:o1', 'noncoding1.6G.%:c2', 'noncoding2.assembled:o1', 'noncoding2.match:o1', 'noncoding2.ref_seq:o3', 'noncoding2.pct_id:c1', 'noncoding2.ctg_cov:c3', 'noncoding2.known_var:o1', 'noncoding2.novel_var:o1', 'noncoding2.id2:o1', 'noncoding2.id2.%:c2', 'noncoding2.42T:o1', 'noncoding2.42T.%:c2', 'noncoding2.52GT:o1', 'noncoding2.52GT.%:c2', 'presence_absence1.assembled:o1', 'presence_absence1.match:o1', 'presence_absence1.ref_seq:o4', 'presence_absence1.pct_id:c1', 'presence_absence1.ctg_cov:c3', 'presence_absence1.known_var:o1', 'presence_absence1.novel_var:o1', 'presence_absence1.A10V:o1']
        expected_csv_header = ['name', 'noncoding1.assembled', 'noncoding1.match', 'noncoding1.ref_seq', 'noncoding1.pct_id', 'noncoding1.ctg_cov', 'noncoding1.known_var', 'noncoding1.novel_var', 'noncoding1.id1', 'noncoding1.id1.%', 'noncoding1.id3', 'noncoding1.id3.%', 'noncoding1.14GT', 'noncoding1.14GT.%', 'noncoding1.14T', 'noncoding1.14T.%', 'noncoding1.6G', 'noncoding1.6G.%', 'noncoding2.assembled', 'noncoding2.match', 'noncoding2.ref_seq', 'noncoding2.pct_id', 'noncoding2.ctg_cov', 'noncoding2.known_var', 'noncoding2.novel_var', 'noncoding2.id2', 'noncoding2.id2.%', 'noncoding2.42T', 'noncoding2.42T.%', 'noncoding2.52GT', 'noncoding2.52GT.%', 'presence_absence1.assembled', 'presence_absence1.match', 'presence_absence1.ref_seq', 'presence_absence1.pct_id', 'presence_absence1.ctg_cov', 'presence_absence1.known_var', 'presence_absence1.novel_var', 'presence_absence1.A10V']
        expected_matrix = [
            ['sample1', 'yes', 'yes', 'noncoding_ref1', '98.33', '10.0', 'yes', 'no', 'yes', 100.0, 'no', 'NA', 'no', 'NA', 'yes', 100.0, 'no', 'NA', 'yes', 'yes', 'noncoding_ref2', '98.33', '10.0', 'yes', 'no', 'yes_multi_het', 'NA', 'yes', 100.0, 'het', 40.0, 'yes', 'yes', 'presence_absence_ref1', '98.96', '20.1', 'no', 'yes', 'yes'],
            [infiles[1], 'yes', 'yes', 'noncoding_ref1', '98.33', '50.1', 'yes', 'no', 'het', 80.0, 'yes', 100.0, 'het', 80.0, 'no', 'NA', 'yes', 100.0, 'yes', 'yes', 'noncoding_ref2', '98.33', '10.0', 'yes', 'no', 'het', 40.0, 'no', 'NA', 'het', 40.0, 'yes', 'yes', 'presence_absence1', '98.96', '51.1', 'no', 'yes', 'yes']
        ]

        self.assertEqual(expected_phandango_header, got_phandango_header)
        self.assertEqual(expected_csv_header, got_csv_header)
        self.assertEqual(expected_matrix, got_matrix)


    def test_to_matrix_with_groups(self):
        '''Test _to_matrix with groups'''
        infiles = [
            os.path.join(data_dir, 'summary_to_matrix.1.tsv'),
            os.path.join(data_dir, 'summary_to_matrix.2.tsv')
        ]

        s = summary.Summary('out', filenames=infiles, show_var_groups=True)
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        got_phandango_header, got_csv_header, got_matrix = summary.Summary._to_matrix(s.filenames, s.all_data, s.all_potential_columns, s.cluster_columns)

        expected_phandango_header = ['name', 'noncoding1.assembled:o1', 'noncoding1.match:o1', 'noncoding1.ref_seq:o2', 'noncoding1.pct_id:c1', 'noncoding1.ctg_cov:c3', 'noncoding1.known_var:o1', 'noncoding1.novel_var:o1', 'noncoding1.id1:o1', 'noncoding1.id1.%:c2', 'noncoding1.id3:o1', 'noncoding1.id3.%:c2', 'noncoding2.assembled:o1', 'noncoding2.match:o1', 'noncoding2.ref_seq:o3', 'noncoding2.pct_id:c1', 'noncoding2.ctg_cov:c3', 'noncoding2.known_var:o1', 'noncoding2.novel_var:o1', 'noncoding2.id2:o1', 'noncoding2.id2.%:c2', 'presence_absence1.assembled:o1', 'presence_absence1.match:o1', 'presence_absence1.ref_seq:o4', 'presence_absence1.pct_id:c1', 'presence_absence1.ctg_cov:c3', 'presence_absence1.known_var:o1', 'presence_absence1.novel_var:o1']
        expected_csv_header = ['name', 'noncoding1.assembled', 'noncoding1.match', 'noncoding1.ref_seq', 'noncoding1.pct_id', 'noncoding1.ctg_cov', 'noncoding1.known_var', 'noncoding1.novel_var', 'noncoding1.id1', 'noncoding1.id1.%', 'noncoding1.id3', 'noncoding1.id3.%', 'noncoding2.assembled', 'noncoding2.match', 'noncoding2.ref_seq', 'noncoding2.pct_id', 'noncoding2.ctg_cov', 'noncoding2.known_var', 'noncoding2.novel_var', 'noncoding2.id2', 'noncoding2.id2.%', 'presence_absence1.assembled', 'presence_absence1.match', 'presence_absence1.ref_seq', 'presence_absence1.pct_id', 'presence_absence1.ctg_cov', 'presence_absence1.known_var', 'presence_absence1.novel_var']
        expected_matrix = [
            [infiles[0], 'yes', 'yes', 'noncoding_ref1', '98.33', '10.0', 'yes', 'no', 'yes', 100.0, 'no', 'NA', 'yes', 'yes', 'noncoding_ref2', '98.33', '10.0', 'yes', 'no', 'yes_multi_het', 'NA', 'yes', 'yes', 'presence_absence_ref1', '98.96', '20.1', 'no', 'yes'],
            [infiles[1], 'yes', 'yes', 'noncoding_ref1', '98.33', '50.1', 'yes', 'no', 'het', 80.0, 'yes', 100.0, 'yes', 'yes', 'noncoding_ref2', '98.33', '10.0', 'yes', 'no', 'het', 40.0, 'yes', 'yes', 'presence_absence1', '98.96', '51.1', 'no', 'yes']
        ]

        self.assertEqual(expected_phandango_header, got_phandango_header)
        self.assertEqual(expected_csv_header, got_csv_header)
        self.assertEqual(expected_matrix, got_matrix)


    def test_to_matrix_with_vars(self):
        '''Test _to_matrix with vars'''
        infiles = [
            os.path.join(data_dir, 'summary_to_matrix.1.tsv'),
            os.path.join(data_dir, 'summary_to_matrix.2.tsv')
        ]

        s = summary.Summary('out', filenames=infiles, show_known_vars=True, show_novel_vars=True)
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        got_phandango_header, got_csv_header, got_matrix = summary.Summary._to_matrix(s.filenames, s.all_data, s.all_potential_columns, s.cluster_columns)

        expected_phandango_header = ['name', 'noncoding1.assembled:o1', 'noncoding1.match:o1', 'noncoding1.ref_seq:o2', 'noncoding1.pct_id:c1', 'noncoding1.ctg_cov:c3', 'noncoding1.known_var:o1', 'noncoding1.novel_var:o1', 'noncoding1.14GT:o1', 'noncoding1.14GT.%:c2', 'noncoding1.14T:o1', 'noncoding1.14T.%:c2', 'noncoding1.6G:o1', 'noncoding1.6G.%:c2', 'noncoding2.assembled:o1', 'noncoding2.match:o1', 'noncoding2.ref_seq:o3', 'noncoding2.pct_id:c1', 'noncoding2.ctg_cov:c3', 'noncoding2.known_var:o1', 'noncoding2.novel_var:o1', 'noncoding2.42T:o1', 'noncoding2.42T.%:c2', 'noncoding2.52GT:o1', 'noncoding2.52GT.%:c2', 'presence_absence1.assembled:o1', 'presence_absence1.match:o1', 'presence_absence1.ref_seq:o4', 'presence_absence1.pct_id:c1', 'presence_absence1.ctg_cov:c3', 'presence_absence1.known_var:o1', 'presence_absence1.novel_var:o1', 'presence_absence1.A10V:o1']
        expected_csv_header = ['name', 'noncoding1.assembled', 'noncoding1.match', 'noncoding1.ref_seq', 'noncoding1.pct_id', 'noncoding1.ctg_cov', 'noncoding1.known_var', 'noncoding1.novel_var', 'noncoding1.14GT', 'noncoding1.14GT.%', 'noncoding1.14T', 'noncoding1.14T.%', 'noncoding1.6G', 'noncoding1.6G.%', 'noncoding2.assembled', 'noncoding2.match', 'noncoding2.ref_seq', 'noncoding2.pct_id', 'noncoding2.ctg_cov', 'noncoding2.known_var', 'noncoding2.novel_var', 'noncoding2.42T', 'noncoding2.42T.%', 'noncoding2.52GT', 'noncoding2.52GT.%', 'presence_absence1.assembled', 'presence_absence1.match', 'presence_absence1.ref_seq', 'presence_absence1.pct_id', 'presence_absence1.ctg_cov', 'presence_absence1.known_var', 'presence_absence1.novel_var', 'presence_absence1.A10V']
        expected_matrix = [
            [infiles[0], 'yes', 'yes', 'noncoding_ref1', '98.33', '10.0', 'yes', 'no', 'no', 'NA', 'yes', 100.0, 'no', 'NA', 'yes', 'yes', 'noncoding_ref2', '98.33', '10.0', 'yes', 'no', 'yes', 100.0, 'het', 40.0, 'yes', 'yes', 'presence_absence_ref1', '98.96', '20.1', 'no', 'yes', 'yes'],
            [infiles[1], 'yes', 'yes', 'noncoding_ref1', '98.33', '50.1', 'yes', 'no', 'het', 80.0, 'no', 'NA', 'yes', 100.0, 'yes', 'yes', 'noncoding_ref2', '98.33', '10.0', 'yes', 'no', 'no', 'NA', 'het', 40.0, 'yes', 'yes', 'presence_absence1', '98.96', '51.1', 'no', 'yes', 'yes']
        ]

        self.assertEqual(expected_phandango_header, got_phandango_header)
        self.assertEqual(expected_csv_header, got_csv_header)
        self.assertEqual(expected_matrix, got_matrix)


    def test_to_matrix_cluster_only(self):
        '''Test _to_matrix with cluster columns only'''
        infiles = [
            os.path.join(data_dir, 'summary_to_matrix.1.tsv'),
            os.path.join(data_dir, 'summary_to_matrix.2.tsv')
        ]

        s = summary.Summary('out', filenames=infiles)
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        got_phandango_header, got_csv_header, got_matrix = summary.Summary._to_matrix(s.filenames, s.all_data, s.all_potential_columns, s.cluster_columns)

        expected_phandango_header = ['name', 'noncoding1.assembled:o1', 'noncoding1.match:o1', 'noncoding1.ref_seq:o2', 'noncoding1.pct_id:c1', 'noncoding1.ctg_cov:c3', 'noncoding1.known_var:o1', 'noncoding1.novel_var:o1', 'noncoding2.assembled:o1', 'noncoding2.match:o1', 'noncoding2.ref_seq:o3', 'noncoding2.pct_id:c1', 'noncoding2.ctg_cov:c3', 'noncoding2.known_var:o1', 'noncoding2.novel_var:o1', 'presence_absence1.assembled:o1', 'presence_absence1.match:o1', 'presence_absence1.ref_seq:o4', 'presence_absence1.pct_id:c1', 'presence_absence1.ctg_cov:c3', 'presence_absence1.known_var:o1', 'presence_absence1.novel_var:o1']
        expected_csv_header = ['name', 'noncoding1.assembled', 'noncoding1.match', 'noncoding1.ref_seq', 'noncoding1.pct_id', 'noncoding1.ctg_cov', 'noncoding1.known_var', 'noncoding1.novel_var', 'noncoding2.assembled', 'noncoding2.match', 'noncoding2.ref_seq', 'noncoding2.pct_id', 'noncoding2.ctg_cov', 'noncoding2.known_var', 'noncoding2.novel_var', 'presence_absence1.assembled', 'presence_absence1.match', 'presence_absence1.ref_seq', 'presence_absence1.pct_id', 'presence_absence1.ctg_cov', 'presence_absence1.known_var', 'presence_absence1.novel_var']
        expected_matrix = [
            [infiles[0], 'yes', 'yes', 'noncoding_ref1', '98.33', '10.0', 'yes', 'no', 'yes', 'yes', 'noncoding_ref2', '98.33', '10.0', 'yes', 'no', 'yes', 'yes', 'presence_absence_ref1', '98.96', '20.1', 'no', 'yes'],
            [infiles[1], 'yes', 'yes', 'noncoding_ref1', '98.33', '50.1', 'yes', 'no', 'yes', 'yes', 'noncoding_ref2', '98.33', '10.0', 'yes', 'no', 'yes', 'yes', 'presence_absence1', '98.96', '51.1', 'no', 'yes']
        ]

        self.assertEqual(expected_phandango_header, got_phandango_header)
        self.assertEqual(expected_csv_header, got_csv_header)
        self.assertEqual(expected_matrix, got_matrix)


    def test_to_matrix_assembled_only(self):
        '''Test _to_matrix with assembled column only'''
        infiles = [
            os.path.join(data_dir, 'summary_to_matrix.1.tsv'),
            os.path.join(data_dir, 'summary_to_matrix.2.tsv')
        ]

        s = summary.Summary('out', filenames=infiles, cluster_cols='assembled')
        s.samples = summary.Summary._load_input_files(s.filenames, 90)
        s._gather_unfiltered_output_data()
        got_phandango_header, got_csv_header, got_matrix = summary.Summary._to_matrix(s.filenames, s.all_data, s.all_potential_columns, s.cluster_columns)

        expected_phandango_header = ['name', 'noncoding1.assembled:o1', 'noncoding2.assembled:o1', 'presence_absence1.assembled:o1']
        expected_csv_header = ['name', 'noncoding1.assembled', 'noncoding2.assembled', 'presence_absence1.assembled']
        expected_matrix = [
            [infiles[0], 'yes', 'yes', 'yes'],
            [infiles[1], 'yes', 'yes', 'yes']
        ]

        self.assertEqual(expected_phandango_header, got_phandango_header)
        self.assertEqual(expected_csv_header, got_csv_header)
        self.assertEqual(expected_matrix, got_matrix)


    def test_filter_matrix_rows(self):
        '''Test _filter_matrix_rows'''
        matrix = [
            ['yes', 'yes'],
            ['yes', 'no'],
            ['no', 'no'],
            ['yes_nonunique', 'no'],
            ['NA', 'no'],
            ['no', 'NA'],
            ['NA', 'NA']
        ]

        expected = [
            ['yes', 'yes'],
            ['yes', 'no'],
            ['yes_nonunique', 'no'],
        ]
        got = summary.Summary._filter_matrix_rows(matrix)
        self.assertEqual(expected, got)


    def test_filter_matrix_columns(self):
        '''Test _filter_matrix_columns'''
        matrix = [
            ['yes', 'yes', 'no', 'yes_nonunique', 'NA', 'no', 'NA'],
            ['yes', 'no', 'no', 'no', 'no', 'NA', 'NA']
        ]
        phandango_header = ['p1', 'p2', 'p3', 'p4', 'p5', 'p6', 'p7']
        csv_header = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'h7']

        got_phandago_header, got_csv_header, got_matrix  = summary.Summary._filter_matrix_columns(matrix, phandango_header, csv_header)
        expected_phandango_header = ['p1', 'p2', 'p4']
        expected_csv_header = ['h1', 'h2', 'h4']
        expected_matrix = [
            ['yes', 'yes', 'yes_nonunique'],
            ['yes', 'no', 'no'],
        ]
        self.assertEqual(expected_phandango_header, got_phandago_header)
        self.assertEqual(expected_csv_header, got_csv_header)
        self.assertEqual(expected_matrix, got_matrix)


    def test_add_phandango_colour_columns(self):
        '''Test _add_phandango_colour_columns'''
        header = ['head1', 'head2:o1', 'head3:o1', 'head4', 'head5:o1']
        matrix = [
            ['yes', 'yes', 'yes_nonunique', 'yes', 'no'],
            ['yes', 'yes_nonunique', 'no', 'yes', 'NA'],
            ['yes', 'no', 'NA', 'yes', 'yes'],
            ['yes', 'NA', 'yes', 'yes', 'yes_nonunique'],
        ]

        expected_header = ['head1', 'head2', 'head2:colour', 'head3', 'head3:colour', 'head4', 'head5', 'head5:colour']
        expected_matrix = [
            ['yes', 'yes', '#33a02c', 'yes_nonunique', '#b2df8a', 'yes', 'no', '#fb9a99'],
            ['yes', 'yes_nonunique', '#b2df8a', 'no', '#fb9a99', 'yes', 'NA', '#ffffff'],
            ['yes', 'no', '#fb9a99', 'NA', '#ffffff', 'yes', 'yes', '#33a02c'],
            ['yes', 'NA', '#ffffff', 'yes', '#33a02c', 'yes', 'yes_nonunique', '#b2df8a']
        ]
        got_header, got_matrix = summary.Summary._add_phandango_colour_columns(header, matrix, nonunique_same_colour=False)
        self.assertEqual(expected_header, got_header)
        self.assertEqual(expected_matrix, got_matrix)

        expected_matrix = [
            ['yes', 'yes', '#33a02c', 'yes_nonunique', '#33a02c', 'yes', 'no', '#fb9a99'],
            ['yes', 'yes_nonunique', '#33a02c', 'no', '#fb9a99', 'yes', 'NA', '#ffffff'],
            ['yes', 'no', '#fb9a99', 'NA', '#ffffff', 'yes', 'yes', '#33a02c'],
            ['yes', 'NA', '#ffffff', 'yes', '#33a02c', 'yes', 'yes_nonunique', '#33a02c']
        ]
        got_header, got_matrix = summary.Summary._add_phandango_colour_columns(header, matrix, nonunique_same_colour=True)
        self.assertEqual(expected_header, got_header)
        self.assertEqual(expected_matrix, got_matrix)


    def test_matrix_to_csv(self):
        '''Test _matrix_to_csv'''
        matrix = [
            ['line1_1', 'line1_2'],
            ['line2_1', 'line2_2'],
        ]
        header = ['head1', 'head2']
        tmpfile = 'tmp.test.matrix_to_csv.csv'
        summary.Summary._matrix_to_csv(matrix, header, tmpfile)
        with open(tmpfile) as f:
            got = f.read()

        expected = 'head1,head2\nline1_1,line1_2\nline2_1,line2_2\n'
        self.assertEqual(expected, got)
        os.unlink(tmpfile)


    def test_matrix_to_csv_remove_nas(self):
        '''Test _matrix_to_csv with remove_nas '''
        matrix = [
            ['line1_1', 'line1_2', 'NA', 'foo'],
            ['NA', 'NA', 'bar', 'NA'],
        ]
        header = ['head1', 'head2', 'head3', 'head4']
        tmpfile = 'tmp.test.matrix_to_csv_remove_nas.csv'
        summary.Summary._matrix_to_csv(matrix, header, tmpfile, remove_nas=True)
        with open(tmpfile) as f:
            got = f.read()

        expected = 'head1,head2,head3,head4\nline1_1,line1_2,,foo\n,,bar,\n'
        self.assertEqual(expected, got)
        os.unlink(tmpfile)


    def test_distance_score_bewteen_values(self):
        '''Test _distance_score_bewteen_values'''
        tests = [
            (('no', 'no'), 0),
            (('no', 'yes'), 1),
            (('no', 'yes_nonunique'), 1),
            (('no', 'fragmented'), 1),
            (('yes', 'no'), 1),
            (('yes', 'yes'), 0),
            (('yes', 'yes_nonunique'), 1),
            (('yes', 'fragmented'), 1),
            (('yes_nonunique', 'no'), 1),
            (('yes_nonunique', 'yes'), 1),
            (('yes_nonunique', 'yes_nonunique'), 0),
            (('yes_nonunique', 'fragmented'), 1),
            (('fragmented', 'no'), 1),
            (('fragmented', 'yes'), 1),
            (('fragmented', 'yes_nonunique'), 1),
            (('fragmented', 'fragmented'), 0),
            (('NA', 'no'), 0),
            (('NA', 'yes'), 1),
            (('NA', 'yes_nonunique'), 1),
            (('NA', 'fragmented'), 1),
        ]

        for (val1, val2), expected in tests:
            self.assertEqual(expected, summary.Summary._distance_score_between_values(val1, val2))
            self.assertEqual(expected, summary.Summary._distance_score_between_values(val2, val1))



    def test_distance_score_between_lists(self):
        '''Test _distance_score_between_lists'''
        list1 = ['NA', 'no', 'yes']
        list2 = ['NA', 'no', 'no']
        self.assertEqual(1, summary.Summary._distance_score_between_lists(list1, list2))


    def test_write_distance_matrix(self):
        '''Test _write_distance_matrix'''
        rows = [
            ['file1', 'no', 'yes', 'no'],
            ['file2', 'yes', 'no', 'yes'],
            ['file3', 'no', 'no', 'yes'],
        ]

        tmp_distances = 'tmp.test.write_distance_matrix.distances'
        summary.Summary._write_distance_matrix(rows, tmp_distances)
        expected = os.path.join(data_dir, 'summary_test_write_distance_matrix.distances')
        self.assertTrue(filecmp.cmp(expected, tmp_distances, shallow=False))
        os.unlink(tmp_distances)


    def test_newick_from_dist_matrix(self):
        '''Test _newick_from_dist_matrix'''
        tmp_tree = 'tmp.test.newick_from_dist_matrix.tre'
        dist_file = os.path.join(data_dir, 'summary_test_newick_from_dist_matrix.distances')
        summary.Summary._newick_from_dist_matrix(dist_file, tmp_tree)
        # the exact ordering of the nodes is not predictable, so we'll trust dendropy
        # and just check that an output file got written
        self.assertTrue(os.path.exists(tmp_tree))
        os.unlink(tmp_tree)


    def test_whole_run(self):
        '''Test whole run to check csv ok (skip making tree)'''
        tmp_out = 'tmp.summary_test_whole_run.out'''
        infiles = [
            os.path.join(data_dir, 'summary_test_whole_run.in.1.tsv'),
            os.path.join(data_dir, 'summary_test_whole_run.in.2.tsv'),
        ]

        s = summary.Summary(
            tmp_out,
            filenames=infiles,
            make_phandango_tree=False,
            show_var_groups=True,
            show_known_vars=True,
            show_novel_vars=True
        )

        s.run()
        expected_file = os.path.join(data_dir, 'summary_test_whole_run.out.csv')
        # we don't know the full path of the input files, so check all the other columns
        with open(expected_file) as f:
            expected = [line.rstrip().split(',', maxsplit=1)[1] for line in f]
        with open(tmp_out + '.csv') as f:
            got = [line.rstrip().split(',', maxsplit=1)[1] for line in f]

        self.assertEqual(expected, got)
        os.unlink(tmp_out + '.csv')
        os.unlink(tmp_out + '.phandango.csv')



import unittest
import os
import pysam
import pyfastaq
from ariba import mapping, external_progs

modules_dir = os.path.dirname(os.path.abspath(mapping.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')
extern_progs = external_progs.ExternalProgs()


# different smalt version output slightly different BAMs. Some columns
# should never change, so check just those ones
def get_sam_columns(bamfile):
    sams = []
    sam_reader = pysam.Samfile(bamfile, "rb")
    for sam in sam_reader.fetch(until_eof=True):
        if sam.is_unmapped:
            refname = None
        else:
            refname = sam_reader.getrname(sam.tid)
        sams.append((sam.qname, sam.flag, refname, sam.pos, sam.cigar, sam.seq))
    return sams


class TestMapping(unittest.TestCase):
    def test_bowtie2_index(self):
        '''test bowtie2_index'''
        tmp_ref = 'tmp.test_bowtie2_index.ref.fa'
        with open(tmp_ref, 'w') as f:
            print('>ref', file=f)
            print('ATCATACTACTCATACTGACTCATCATCATCATGACGTATG', file=f)

        tmp_out = 'tmp.test_bowtie2_index.ref.out'
        mapping.bowtie2_index(tmp_ref, tmp_out, bowtie2=extern_progs.exe('bowtie2'))

        expected_files = [tmp_out + '.' + x + '.bt2' for x in ['1', '2', '3', '4', 'rev.1', 'rev.2']]
        for filename in expected_files:
            self.assertTrue(os.path.exists(filename))
            os.unlink(filename)

        os.unlink(tmp_ref)


    def test_run_bowtie2(self):
        '''Test run_bowtie2 unsorted'''
        self.maxDiff = None
        ref = os.path.join(data_dir, 'mapping_test_bowtie2_ref.fa')
        reads1 = os.path.join(data_dir, 'mapping_test_bowtie2_reads_1.fq')
        reads2 = os.path.join(data_dir, 'mapping_test_bowtie2_reads_2.fq')
        out_prefix = 'tmp.out.bowtie2'
        mapping.run_bowtie2(
            reads1,
            reads2,
            ref,
            out_prefix,
            bowtie2=extern_progs.exe('bowtie2'),
            bowtie2_version=extern_progs.version('bowtie2'),
        )
        expected = get_sam_columns(os.path.join(data_dir, 'mapping_test_bowtie2_unsorted.bam'))
        got = get_sam_columns(out_prefix + '.bam')
        self.assertListEqual(expected, got)
        os.unlink(out_prefix + '.bam')


    def test_run_bowtie2_remove_both_unmapped(self):
        '''Test run_bowtie2 unsorted remove both unmapped'''
        self.maxDiff = None
        ref = os.path.join(data_dir, 'mapping_test_bowtie2_ref.fa')
        reads1 = os.path.join(data_dir, 'mapping_test_bowtie2_remove_both_unmapped_reads_1.fq')
        reads2 = os.path.join(data_dir, 'mapping_test_bowtie2_remove_both_unmapped_reads_2.fq')
        out_prefix = 'tmp.out.bowtie2_remove_both_unmapped'
        mapping.run_bowtie2(
            reads1,
            reads2,
            ref,
            out_prefix,
            bowtie2=extern_progs.exe('bowtie2'),
            bowtie2_version=extern_progs.version('bowtie2'),
            remove_both_unmapped=True,
        )
        expected = get_sam_columns(os.path.join(data_dir, 'mapping_test_bowtie2_remove_both_unmapped_reads.bam'))
        got = get_sam_columns(out_prefix + '.bam')
        self.assertListEqual(expected, got)
        os.unlink(out_prefix + '.bam')


    def test_run_bowtie2_and_sort(self):
        '''Test run_bowtie2 sorted'''
        ref = os.path.join(data_dir, 'mapping_test_bowtie2_ref.fa')
        reads1 = os.path.join(data_dir, 'mapping_test_bowtie2_reads_1.fq')
        reads2 = os.path.join(data_dir, 'mapping_test_bowtie2_reads_2.fq')
        out_prefix = 'tmp.out.bowtie2'
        mapping.run_bowtie2(
            reads1,
            reads2,
            ref,
            out_prefix,
            sort=True,
            bowtie2=extern_progs.exe('bowtie2'),
            bowtie2_version=extern_progs.version('bowtie2'),
        )
        expected = get_sam_columns(os.path.join(data_dir, 'mapping_test_bowtie2_sorted.bam'))
        got = get_sam_columns(out_prefix + '.bam')
        self.assertListEqual(expected, got)
        os.unlink(out_prefix + '.bam')
        os.unlink(out_prefix + '.bam.bai')


    def test_get_total_alignment_score(self):
        '''Test get_total_alignment_score'''
        bam = os.path.join(data_dir, 'mapping_test_get_total_alignment_score.bam')
        expected = 219
        got = mapping.get_total_alignment_score(bam)
        self.assertEqual(got, expected)


    def test_sam_to_fastq(self):
        '''test sam_to_fastq'''
        expected = [
            pyfastaq.sequences.Fastq('read1/1', 'GTATGAGTAGATATAAAGTCCGGAACTGTGATCGGGGGCGATTTATTTACTGGCCGTCCC', 'GHIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII'),
            pyfastaq.sequences.Fastq('read1/2', 'TCCCATACGTTGCAATCTGCAGACGCCACTCTTCCACGTCGGACGAACGCAACGTCAGGA', 'IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIHGEDCBA')
        ]


        sam_reader = pysam.Samfile(os.path.join(data_dir, 'mapping_test_sam_to_fastq.bam'), "rb")
        i = 0
        for s in sam_reader.fetch(until_eof=True):
            self.assertEqual(expected[i], mapping.sam_to_fastq(s))
            i += 1


    def test_sam_pair_to_insert(self):
        '''test sam_pair_to_insert'''
        expected = [
            None, # both unmapped
            None, # read 1 unmapped
            None, # read 2 unmpapped
            None, # mapped to different seqs
            None, # same seqs, wrond orientation
            660
        ]

        sam1 = None
        i = 0
        sam_reader = pysam.Samfile(os.path.join(data_dir, 'mapping_test_sam_pair_to_insert.bam'), 'rb')
        for s in sam_reader.fetch(until_eof=True):
            if sam1 is None:
                sam1 = s
                continue

            self.assertEqual(mapping.sam_pair_to_insert(s, sam1), expected[i])
            sam1 = None
            i += 1





import os
import copy
import pyfastaq
import pymummer

class Error (Exception): pass

class AssemblyCompare:
    def __init__(self,
      assembly_fa,
      assembly_sequences,
      ref_fa,
      ref_sequence,
      outprefix,
      refdata,
      nucmer_min_id=90,
      nucmer_min_len=20,
      nucmer_breaklen=200,
      assembled_threshold=0.95,
      unique_threshold=0.03,
      max_gene_nt_extend=30,
    ):
        self.assembly_fa = os.path.abspath(assembly_fa)
        self.assembly_sequences = assembly_sequences
        self.ref_fa = os.path.abspath(ref_fa)
        self.ref_sequence = ref_sequence
        self.outprefix = os.path.abspath(outprefix)
        self.refdata = refdata

        self.nucmer_min_id = nucmer_min_id
        self.nucmer_min_len = nucmer_min_len
        self.nucmer_breaklen = nucmer_breaklen
        self.assembled_threshold = assembled_threshold
        self.unique_threshold = unique_threshold
        self.max_gene_nt_extend = max_gene_nt_extend
        self.scaff_name_matching_ref = None
        self.gene_matching_ref = None
        self.gene_matching_ref_type = None
        self.gene_start_bases_added = None
        self.gene_end_bases_added = None

        self.nucmer_coords_file = self.outprefix + '.nucmer.coords'
        self.nucmer_snps_file = self.nucmer_coords_file + '.snps'


    def _run_nucmer(self):
        pymummer.nucmer.Runner(
            self.ref_fa,
            self.assembly_fa,
            self.nucmer_coords_file,
            min_id=self.nucmer_min_id,
            min_length=self.nucmer_min_len,
            breaklen=self.nucmer_breaklen,
            maxmatch=True,
            show_snps=True,
            show_snps_C=False,
        ).run()


    @staticmethod
    def _parse_nucmer_coords_file(coords_file, ref_name):
        '''Input is coords file made by self._run_nucmer. Reference should have one sequence only.
           ref_name is name fo the reference sequence, to sanity check the coords file.
           Returns dictionary. Key = assembly contig name. Value = list of nucmer hits to that contig'''
        file_reader = pymummer.coords_file.reader(coords_file)
        nucmer_hits = {}
        for hit in file_reader:
            assert hit.ref_name == ref_name
            contig = hit.qry_name
            if contig not in nucmer_hits:
                nucmer_hits[contig] = []
            nucmer_hits[contig].append(copy.copy(hit))

        return nucmer_hits


    @staticmethod
    def _nucmer_hits_to_percent_identity(nucmer_hits):
        '''Input is hits made by self._parse_nucmer_coords_file.
           Returns dictionary. key = contig name. Value = percent identity of hits to that contig'''
        percent_identities = {}
        max_lengths = {}

        for contig in nucmer_hits:
            max_length = -1
            percent_identity = 0
            for hit in nucmer_hits[contig]:
                if hit.hit_length_qry > max_length:
                    max_length = hit.hit_length_qry
                    percent_identity = hit.percent_identity
            percent_identities[contig] = percent_identity

        return percent_identities


    @staticmethod
    def _nucmer_hits_to_assembly_coords(nucmer_hits):
        '''Input is hits made by self._parse_nucmer_coords_file.
           Returns dictionary. key = contig name. Value = list of coords that match
           to the reference gene'''
        coords = {}
        for l in nucmer_hits.values():
            for hit in l:
                if hit.qry_name not in coords:
                    coords[hit.qry_name] = []
                coords[hit.qry_name].append(hit.qry_coords())

        for scaff in coords:
            pyfastaq.intervals.merge_overlapping_in_list(coords[scaff])

        return coords


    def assembly_match_coords(self):
        return self._nucmer_hits_to_assembly_coords(self.nucmer_hits)


    @classmethod
    def nucmer_hits_to_ref_coords(cls, nucmer_hits, contig=None):
        '''Input is hits made by self._parse_nucmer_coords_file.
           Returns dictionary. Key = contig name. Value = list of coords in the
           reference sequence for that contig.
           if contig=contig_name, then just gets the ref coords from that contig,
           instead of using all the contigs'''
        coords = []
        if contig is None:
            coords = {key: [] for key in nucmer_hits.keys()}
        else:
            coords = {contig: []}

        for key in coords:
            coords[key] = [hit.ref_coords() for hit in nucmer_hits[key]]
            pyfastaq.intervals.merge_overlapping_in_list(coords[key])

        return coords


    @classmethod
    def nucmer_hits_to_ref_and_qry_coords(cls, nucmer_hits, contig=None):
        '''Same as nucmer_hits_to_ref_coords, except removes containing hits first,
           and returns ref and qry coords lists'''
        if contig is None:
            ctg_coords = {key: [] for key in nucmer_hits.keys()}
        else:
            ctg_coords = {contig: []}

        ref_coords = {}

        for key in ctg_coords:
            hits = copy.copy(nucmer_hits[key])
            hits.sort(key=lambda x: len(x.ref_coords()))

            if len(hits) > 1:
                i = 0
                while i < len(hits) - 1:
                    c1 = hits[i].ref_coords()
                    c2 = hits[i+1].ref_coords()
                    if c2.contains(c1):
                        hits.pop(i)
                    else:
                        i += 1

            ref_coords[key] = [hit.ref_coords() for hit in hits]
            ctg_coords[key] = [hit.qry_coords() for hit in hits]
            pyfastaq.intervals.merge_overlapping_in_list(ref_coords[key])
            pyfastaq.intervals.merge_overlapping_in_list(ctg_coords[key])

        return ctg_coords, ref_coords


    @staticmethod
    def ref_cov_per_contig(nucmer_hits):
        '''Input is hits made by self._parse_nucmer_coords_file.
           Returns dictionary. key = contig name. Value = number of bases that
           match to the reference sequence.'''
        coords = AssemblyCompare.nucmer_hits_to_ref_coords(nucmer_hits)
        return {x: pyfastaq.intervals.length_sum_from_list(coords[x]) for x in coords}


    @staticmethod
    def _get_assembled_reference_sequences(nucmer_hits, ref_sequence, assembly):
        '''nucmer_hits =  hits made by self._parse_nucmer_coords_file.
           ref_gene = reference sequence (pyfastaq.sequences.Fasta object)
           assembly = dictionary of contig name -> contig.
           Makes a set of Fasta objects of each piece of assembly that
           corresponds to the reference sequeunce.'''
        sequences = {}

        for contig in sorted(nucmer_hits):
            for hit in nucmer_hits[contig]:
                qry_coords = hit.qry_coords()
                fa = assembly[hit.qry_name].subseq(qry_coords.start, qry_coords.end + 1)
                if hit.on_same_strand():
                    strand = '+'
                else:
                    fa.revcomp()
                    strand = '-'
                ref_coords = hit.ref_coords()
                fa.id = '.'.join([
                    ref_sequence.id,
                    str(ref_coords.start + 1),
                    str(ref_coords.end + 1),
                    contig,
                    str(qry_coords.start + 1),
                    str(qry_coords.end + 1),
                    strand
                ])

                if hit.hit_length_ref == hit.ref_length:
                    fa.id += '.complete'

                sequences[fa.id] = fa

        return sequences


    @staticmethod
    def _whole_gene_covered_by_nucmer_hits(nucmer_hits, ref_seq, percent_threshold, max_nt_extend):
        '''Returns true iff the reference sequence is covered by nucmer hits.
           nucmer_hits = hits made by self._parse_nucmer_coords_file.
           Counts as covered if (total ref bases covered) / len(ref_seq) >= threshold'''
        coords = AssemblyCompare.nucmer_hits_to_ref_coords(nucmer_hits)
        covered = []
        for coords_list in coords.values():
            covered.extend(coords_list)
        pyfastaq.intervals.merge_overlapping_in_list(covered)
        return (2 * max_nt_extend + pyfastaq.intervals.length_sum_from_list(covered)) / len(ref_seq) >= percent_threshold


    @staticmethod
    def _ref_has_region_assembled_twice(nucmer_hits, ref_seq, threshold):
        '''Returns true iff there is a part of the reference that is assembled
           more than once (ie covered by >1 nucmer hit).
           Needs a minimum proportin of the ref to be assembled more than once,
           determined by threshold.
           nucmer_hits = hits made by self._parse_nucmer_coords_file.'''
        coords = AssemblyCompare.nucmer_hits_to_ref_coords(nucmer_hits)
        covered = []
        for coords_list in coords.values():
            covered.extend(coords_list)
        covered.sort()

        if len(covered) <= 1:
            return False

        coverage = {}
        for i in covered:
            for j in range(i.start, i.end + 1):
                coverage[j] = coverage.get(j, 0) + 1

        bases_depth_at_least_two = len([1 for x in coverage.values() if x > 1])
        return bases_depth_at_least_two / len(ref_seq) >= threshold


    @classmethod
    def _longest_nucmer_hit_in_ref(cls, nucmer_hits):
        max_length = None
        max_hit = None

        for l in nucmer_hits.values():
            for hit in l:
                if max_length is None or hit.hit_length_ref > max_length:
                    max_length = hit.hit_length_ref
                    max_hit = hit

        return max_hit


    @classmethod
    def _find_previous_start_codon(cls, sequence, start_coord, min_start_coord):
        for i in range(start_coord, min_start_coord - 1, -3):
            codon = pyfastaq.sequences.Fasta('x', sequence[i:i+3])
            aa = codon.translate()
            if aa.seq == '*':
                return None
            elif codon.seq in pyfastaq.genetic_codes.starts[pyfastaq.sequences.genetic_code]:
                return i

        return None


    @classmethod
    def _find_next_stop_codon(cls, sequence, end_coord, max_end_coord):
        final_i = min(len(sequence) - 3, max_end_coord - 2)
        for i in range(end_coord, final_i + 1, 3):
            codon = pyfastaq.sequences.Fasta('x', sequence[i:i+3])
            aa = codon.translate()
            if aa.seq == '*':
                return i

        return None


    @classmethod
    def _gene_from_nucmer_match(cls, nucmer_match, contig, max_end_nt_extend):
        if nucmer_match.on_same_strand():
            revcomp = False
        else:
            revcomp = True
            nucmer_match = copy.copy(nucmer_match)
            nucmer_match.reverse_query()
            contig = copy.copy(contig)
            contig.revcomp()

        ref_hit_start = min(nucmer_match.ref_start, nucmer_match.ref_end)
        contig_hit_start = min(nucmer_match.qry_start, nucmer_match.qry_end)
        min_allowed_start = max(0, contig_hit_start - max_end_nt_extend)
        if ref_hit_start % 3 != 0:
            contig_hit_start += 3 - (ref_hit_start % 3)
        contig_hit_end = max(nucmer_match.qry_start, nucmer_match.qry_end)
        max_allowed_end = min(len(contig) - 1, contig_hit_end + max_end_nt_extend)
        contig_hit_end -= (contig_hit_end - contig_hit_start + 1) % 3
        assert contig_hit_start < contig_hit_end
        gene_nt_name = nucmer_match.qry_name + '.' + str(contig_hit_start + 1) + '-' + str(contig_hit_end + 1)

        gene_nt = pyfastaq.sequences.Fasta(gene_nt_name, contig[contig_hit_start:contig_hit_end+1])
        assert len(gene_nt) % 3 == 0
        gene_aa = gene_nt.translate()
        if '*' in gene_aa[:-1]:
            return gene_nt, 'HAS_STOP', None, None

        extended_start_position = AssemblyCompare._find_previous_start_codon(contig, contig_hit_start, min_allowed_start)
        extended_end_position = AssemblyCompare._find_next_stop_codon(contig, contig_hit_end - 2, max_allowed_end)
        start = extended_start_position if extended_start_position is not None else contig_hit_start
        end = extended_end_position + 2 if extended_end_position is not None else contig_hit_end

        if revcomp:
            gene_nt_name = nucmer_match.qry_name + '.' + str(end + 1) + '-' + str(start + 1)
        else:
            gene_nt_name = nucmer_match.qry_name + '.' + str(start + 1) + '-' + str(end + 1)

        gene_nt = pyfastaq.sequences.Fasta(gene_nt_name, contig[start:end+1])
        start_nt_added = None if extended_start_position is None else min(nucmer_match.qry_start, nucmer_match.qry_end) - start
        end_nt_added = None if extended_end_position is None else end - max(nucmer_match.qry_start, nucmer_match.qry_end)

        if None in [extended_start_position, extended_end_position]:
            return gene_nt, 'START_OR_END_FAIL', start_nt_added, end_nt_added
        else:
            return gene_nt, 'GENE_FOUND', start_nt_added, end_nt_added


    @staticmethod
    def _get_gene_matching_ref(nucmer_hits, contigs, max_end_nt_extend):
        longest_match = AssemblyCompare._longest_nucmer_hit_in_ref(nucmer_hits)
        if longest_match is None:
            return None, 'NO_MATCH', None, None
        else:
            return (longest_match.qry_name,) + AssemblyCompare._gene_from_nucmer_match(longest_match, contigs[longest_match.qry_name], max_end_nt_extend)


    @staticmethod
    def _ref_covered_by_at_least_one_full_length_contig(nucmer_hits, percent_threshold, max_nt_extend):
        '''Returns true iff there exists a contig that completely
           covers the reference sequence
           nucmer_hits = hits made by self._parse_nucmer_coords_file.'''
        for l in nucmer_hits.values():
            for hit in l:
                if ( (2 * max_nt_extend) + len(hit.ref_coords()) ) / hit.ref_length >= percent_threshold:
                    return True
        return False


    def update_flag(self, flag):
        if self._whole_gene_covered_by_nucmer_hits(self.nucmer_hits, self.ref_sequence, self.assembled_threshold, self.max_gene_nt_extend):
            flag.add('assembled')

        if self.assembled_into_one_contig:
            flag.add('assembled_into_one_contig')

        if self._ref_has_region_assembled_twice(self.nucmer_hits, self.ref_sequence, self.unique_threshold):
            flag.add('region_assembled_twice')

        ref_seq_type, is_variant_only = self.refdata.sequence_type(self.ref_sequence.id)
        if ref_seq_type == 'p' and self.gene_matching_ref_type == 'GENE_FOUND':
            flag.add('complete_gene')

        if len(self.nucmer_hits) == 1:
            flag.add('unique_contig')

        return flag


    @staticmethod
    def nucmer_hit_containing_reference_position(nucmer_hits, ref_name, ref_position, qry_name=None):
        '''Returns the first nucmer match found that contains the given
           reference location. nucmer_hits = hits made by self._parse_nucmer_coords_file.
           Returns None if no matching hit found'''
        for contig_name in nucmer_hits:
            for hit in nucmer_hits[contig_name]:
                if hit.ref_name == ref_name and (qry_name is None or qry_name == hit.qry_name) and hit.ref_coords().distance_to_point(ref_position) == 0:
                    return hit

        return None


    def run(self):
        self._run_nucmer()
        self.nucmer_hits = self._parse_nucmer_coords_file(self.nucmer_coords_file, self.ref_sequence.id)
        self.percent_identities = self._nucmer_hits_to_percent_identity(self.nucmer_hits)
        self.assembled_reference_sequences = self._get_assembled_reference_sequences(self.nucmer_hits, self.ref_sequence, self.assembly_sequences)
        ref_seq_type, is_variant_only = self.refdata.sequence_type(self.ref_sequence.id)
        if self._ref_covered_by_at_least_one_full_length_contig(self.nucmer_hits, self.assembled_threshold, self.max_gene_nt_extend):
            self.assembled_into_one_contig = True
            if ref_seq_type == 'p':
                self.scaff_name_matching_ref, self.gene_matching_ref, self.gene_matching_ref_type, self.gene_start_bases_added, self.gene_end_bases_added = self._get_gene_matching_ref(self.nucmer_hits, self.assembly_sequences, self.max_gene_nt_extend)
        else:
            self.assembled_into_one_contig = False



import unittest
import os
import filecmp
from ariba import pubmlst_getter

modules_dir = os.path.dirname(os.path.abspath(pubmlst_getter.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestPubmlstGetter(unittest.TestCase):
    def setUp(self):
        xml_file = os.path.join(data_dir, 'pubmlst_getter.dbases.xml')
        self.pmlst_getter = pubmlst_getter.PubmlstGetter(xml_file=xml_file)


    def test_get_species_list(self):
        '''test _get_species_list'''
        expected = ['Achromobacter spp.', 'Acinetobacter baumannii#1', 'Acinetobacter baumannii#2', 'Aeromonas spp.']
        got = self.pmlst_getter._get_species_list()
        self.assertEqual(expected, got)


    def test_get_profile_and_fasta_urls(self):
        '''test _get_profile_and_fasta_urls'''
        expected_profile = 'http://pubmlst.org/data/profiles/abaumannii_2.txt'
        expected_fasta = [
            'http://pubmlst.org/data/alleles/abaumannii_2/Pas_cpn60.tfa',
            'http://pubmlst.org/data/alleles/abaumannii_2/Pas_fusA.tfa',
            'http://pubmlst.org/data/alleles/abaumannii_2/Pas_gltA.tfa',
            'http://pubmlst.org/data/alleles/abaumannii_2/Pas_pyrG.tfa',
            'http://pubmlst.org/data/alleles/abaumannii_2/Pas_recA.tfa',
            'http://pubmlst.org/data/alleles/abaumannii_2/Pas_rplB.tfa',
            'http://pubmlst.org/data/alleles/abaumannii_2/Pas_rpoB.tfa',
        ]

        got_profile, got_fasta = self.pmlst_getter._get_profile_and_fasta_urls('Acinetobacter baumannii#2')
        self.assertEqual(expected_profile, got_profile)
        self.assertEqual(expected_fasta, got_fasta)

    def test_rename_seqs_in_fasta(self):
        '''test _rename_seqs_in_fasta'''
        infile = os.path.join(data_dir, 'pubmlst_rename_seqs.in.fa')
        expected_file = os.path.join(data_dir, 'pubmlst_rename_seqs.expected.fa')
        tmp_out = 'tmp.test.pubmlst_rename_seqs.out.fa'
        pubmlst_getter.PubmlstGetter._rename_seqs_in_fasta(infile, tmp_out)
        self.assertTrue(filecmp.cmp(expected_file, tmp_out, shallow=False))
        os.unlink(tmp_out)




import unittest
import os
from ariba import flag

modules_dir = os.path.dirname(os.path.abspath(flag.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')

class TestFlag(unittest.TestCase):
    def test_init_and_to_number(self):
        '''Test __init__ and to_number'''
        for i in range(512):
            f = flag.Flag(i)
            self.assertEqual(f.to_number(), i)


    def test_set_flag(self):
        '''Test set_flag'''
        for i in range(512):
            f = flag.Flag()
            f.set_flag(i)
            self.assertEqual(f.to_number(), i)


    def test_add(self):
        '''Test add'''
        f = flag.Flag()
        expected = [1, 3, 7, 15, 31, 63, 127, 255, 511, 1023, 2047]
        for i in range(len(flag.flags_in_order)):
            f.add(flag.flags_in_order[i])
            self.assertEqual(f.to_number(), expected[i])


    def test_str(self):
        '''Test __str__'''
        for i in range(512):
            f = flag.Flag(i)
            self.assertEqual(str(f), str(i))


    def test_to_long_str(self):
        '''Test to_long_str'''
        f = flag.Flag(13)
        expected = '\n'.join([
            '[X] assembled',
            '[ ] assembled_into_one_contig',
            '[X] region_assembled_twice',
            '[X] complete_gene',
            '[ ] unique_contig',
            '[ ] scaffold_graph_bad',
            '[ ] assembly_fail',
            '[ ] variants_suggest_collapsed_repeat',
            '[ ] hit_both_strands',
            '[ ] has_variant',
            '[ ] ref_seq_choose_fail',
        ])

        self.assertEqual(expected, f.to_long_string())


    def test_has(self):
        '''Test has'''
        for x in flag.flags_in_order:
            f = flag.Flag(0)
            self.assertFalse(f.has(x))
            f.add(x)
            self.assertTrue(f.has(x))


    def test_to_comma_separated_string(self):
        '''Test to_comma_separated_string'''
        f = flag.Flag(27)
        expected = 'assembled,assembled_into_one_contig,complete_gene,unique_contig'
        self.assertEqual(expected, f.to_comma_separated_string())




import tempfile
import re
import time
import os
import urllib.request
import xml.etree.ElementTree as ET
import pyfastaq
from ariba import common

class Error (Exception): pass


class PubmlstGetter:
    def __init__(self, debug=False, xml_file=None, verbose=False):
        self.debug = debug
        self.verbose = verbose

        if xml_file is None:
            self.xml_tree = self._get_xml_file_tree()
        else:
            self.xml_tree = ET.parse(xml_file)


    def _get_xml_file_tree(self):
        xml_url = 'http://pubmlst.org/data/dbases.xml'
        tmpdir = tempfile.mkdtemp(prefix='tmp.get_pubmlst_xml', dir=os.getcwd())
        xml_file = os.path.join(tmpdir, 'out.xml')
        self._download_file(xml_url, xml_file)
        xml_tree = ET.parse(xml_file)

        if not self.debug:
            common.rmtree(tmpdir)

        return xml_tree


    def _download_file(self, url, outfile):
        if self.verbose:
            print('Downloading "', url, '" and saving as "', outfile, '" ...', end='', sep='', flush=True)
        max_attempts = 3
        sleep_time = 3
        for i in range(max_attempts):
            time.sleep(sleep_time)
            try:
                urllib.request.urlretrieve(url, filename=outfile)
            except:
                continue
            break
        else:
            raise Error('Error downloading: ' + url)

        if self.verbose:
            print(' done', flush=True)


    def _get_species_list(self):
        return [x.text.rstrip() for x in self.xml_tree.getroot().findall('species')]


    def _get_profile_and_fasta_urls(self, species):
        species_dict = {x.text.rstrip(): x for x in self.xml_tree.getroot().findall('species')}
        if species not in species_dict:
            raise Error('Error! Species "' + species + '" not found. Cannot continue. Available species:\n' + '\n'.join(sorted(list(species_dict.keys()))))

        try:
            profile_url = species_dict[species].find('mlst').find('database').find('profiles').find('url').text
        except:
            raise Error('Error getting profile url for species ' + species + '. Cannot continue')

        locus_list = species_dict[species].find('mlst').find('database').find('loci').findall('locus')
        fasta_urls = [x.find('url').text for x in locus_list]

        if len(fasta_urls) == 0:
            raise Error('Error! No fasta files found for species ' + species + '. Cannot continue')

        return profile_url, fasta_urls


    @classmethod
    def _rename_seqs_in_fasta(cls, infile, outfile):
        f = pyfastaq.utils.open_file_write(outfile)
        file_reader = pyfastaq.sequences.file_reader(infile)
        nodot_regex = re.compile(r'^.*(?P<separator>[^.0-9])[0-9]+$')

        for seq in file_reader:
            if seq.id.startswith('Oxf.'):
                seq.id = 'Oxf_' + seq.id[4:]

            regex_match = nodot_regex.match(seq.id)
            if regex_match is not None:
                seq.id = '.'.join(seq.id.rsplit(regex_match.groupdict()['separator'], maxsplit=1))

            print(seq, file=f)

        pyfastaq.utils.close(f)


    def _download_profile_and_fastas(self, outdir, profile_url, fasta_urls):
        try:
            os.mkdir(outdir)
        except:
            raise Error('Error mkdir ' + outdir)

        profile_outfile = os.path.join(outdir, 'profile.txt')
        self._download_file(profile_url, profile_outfile)

        for fasta_url in fasta_urls:
            outfile = "{0}.tfa".format(os.path.join(outdir, fasta_url.split('/')[-2]))
            self._download_file(fasta_url, outfile + '.tmp')
            PubmlstGetter._rename_seqs_in_fasta(outfile + '.tmp', outfile)
            os.unlink(outfile + '.tmp')


    def print_available_species(self):
        species_list = self._get_species_list()
        print(*species_list, sep='\n')


    def get_species_files(self, species, outdir):
        profile_url, fasta_urls = self._get_profile_and_fasta_urls(species)
        self._download_profile_and_fastas(outdir, profile_url, fasta_urls)




import unittest
import os
import copy
import pyfastaq
import pysam
from ariba import link

modules_dir = os.path.dirname(os.path.abspath(link.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestLink(unittest.TestCase):
    def test_init_no_links(self):
        '''test link __init__ no links made'''
        sam_reader = pysam.Samfile(os.path.join(data_dir, 'link_test_init.reads.no_link.bam'), "rb")
        ref_lengths = {}
        pyfastaq.tasks.lengths_from_fai(os.path.join(data_dir, 'link_test_init.ref.fa.fai'), ref_lengths)
        for sam in sam_reader.fetch(until_eof=True):
            with self.assertRaises(link.Error):
                link.Link(sam, sam_reader, ref_lengths)


    def test_init_with_link(self):
        '''test link __init__ link made'''
        sam_reader = pysam.Samfile(os.path.join(data_dir, 'link_test_init.reads.make_link.bam'), "rb")
        ref_lengths = {}
        pyfastaq.tasks.lengths_from_fai(os.path.join(data_dir, 'link_test_init.ref.fa.fai'), ref_lengths)
        links_from_bam = []
        for sam in sam_reader.fetch(until_eof=True):
            links_from_bam.append(link.Link(sam, sam_reader, ref_lengths))

        expected = [
            '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '.']),
            '\t'.join(['1', '506', 'R', '.', '2', '500', 'L', '359']),
        ]

        self.assertEqual(len(expected), len(links_from_bam))

        for i in range(len(expected)):
            self.assertEqual(expected[i], str(links_from_bam[i]))


    def test_lt(self):
        '''test __lt__'''
        l = [
            (['1', '10', 'R', '5', '2', '20', 'L', '3'], ['1', '10', 'R', '5', '2', '20', 'L', '3'], False),
            (['1', '10', 'R', '5', '2', '20', 'L', '3'], ['1', '10', 'R', '5', '2', '20', 'L', '4'], True),
            (['1', '10', 'R', '5', '2', '20', 'L', '3'], ['1', '10', 'R', '6', '2', '20', 'L', '3'], True),
            (['1', '10', 'R', '5', '2', '20', 'L', '3'], ['2', '10', 'R', '6', '2', '20', 'L', '3'], True),
            (['2', '10', 'R', '5', '2', '20', 'L', '3'], ['1', '10', 'R', '6', '2', '20', 'L', '3'], False),
        ]

        for t in l:
            link1 = link.Link(None, None, None, '\t'.join(t[0]))
            link2 = link.Link(None, None, None, '\t'.join(t[1]))
            self.assertEqual(link1 < link2, t[2])


    def test_swap(self):
        '''test _swap'''
        original_str = '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '359'])
        original_link = link.Link(None, None, None, s=original_str)
        l = copy.copy(original_link)
        expected_swap = link.Link(None, None, None, s='\t'.join(['2', '500', 'L', '359', '1', '506', 'R', '300']))
        l._swap()
        self.assertEqual(l, expected_swap)
        l._swap()
        self.assertEqual(l, original_link)


    def test_sort(self):
        '''test sort'''
        links = [
            link.Link(None, None, None, s='\t'.join(['ref1', '500', 'L', '359', 'ref2', '506', 'R', '300'])),
            link.Link(None, None, None, s='\t'.join(['ref2', '500', 'L', '359', 'ref1', '506', 'R', '300']))
        ]

        expected = [
            link.Link(None, None, None, s='\t'.join(['ref1', '500', 'L', '359', 'ref2', '506', 'R', '300'])),
            link.Link(None, None, None, s='\t'.join(['ref1', '506', 'R', '300', 'ref2', '500', 'L', '359'])),
        ]

        assert len(links) == len(expected)
        for i in range(len(links)):
            links[i].sort()
            self.assertEqual(links[i], expected[i])


    def test_merge(self):
        '''test merge'''
        link1 = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '.']))
        link2 = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '.', '2', '500', 'L', '359']))
        merged = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '359']))
        with self.assertRaises(link.Error):
            link1.merge(link1)
            link2.merge(link2)

        link1.merge(link2)
        self.assertEqual(link1, merged)


    def test_distance_to_contig_end(self):
        '''test _distance_to_contig_end'''
        links = [
            link.Link(None, None, None, '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '.'])),
            link.Link(None, None, None, '\t'.join(['1', '506', 'R', '.', '2', '500', 'L', '359']))
        ]

        self.assertEqual(links[0]._distance_to_contig_end(1), 205)
        self.assertEqual(links[1]._distance_to_contig_end(2), 359)
        with self.assertRaises(link.Error):
            links[0]._distance_to_contig_end(2)

        with self.assertRaises(link.Error):
            links[1]._distance_to_contig_end(1)


    def test_distance_to_contig_ends(self):
        '''test _distance_to_contig_ends'''
        link1 = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '.']))
        link2 = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '.', '2', '500', 'L', '359']))
        link3 = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '359']))

        with self.assertRaises(link.Error):
            link1._distance_to_contig_ends()

        with self.assertRaises(link.Error):
            link2._distance_to_contig_ends()

        self.assertEqual(link3._distance_to_contig_ends(), (205, 359))


    def test_insert_size(self):
        '''test insert_size'''
        link1 = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '.']))
        link2 = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '.', '2', '500', 'L', '359']))
        link3 = link.Link(None, None, None, '\t'.join(['1', '506', 'R', '300', '2', '500', 'L', '359']))

        with self.assertRaises(link.Error):
            link1.insert_size()

        with self.assertRaises(link.Error):
            link2.insert_size()

        self.assertEqual(link3.insert_size(), 564)




import re
import pyfastaq

class Error (Exception): pass

name_regex = re.compile(r'^(?P<vfdb_id>V\S+\)) \((?P<name>\S+)\) (?P<description>.*\]) \[(?P<genus_etc>.*)\]$')

class VfdbParser:
    def __init__(self, infile, outprefix):
        self.infile = infile
        self.outprefix = outprefix


    @classmethod
    def _fa_header_to_name_pieces(cls, fa_header):
        m = name_regex.search(fa_header)
        if m is None:
            return None
        else:
            return tuple([m.group(x) for x in ['vfdb_id', 'name', 'description', 'genus_etc']])


    @staticmethod
    def _fa_header_to_name_and_metadata(fa_header):
        name_data = VfdbParser._fa_header_to_name_pieces(fa_header)
        if name_data is None:
            return fa_header, '.'
        else:
            vfdb_id, name, description, genus_etc = name_data
            return name + '.' + vfdb_id + '.' + genus_etc.replace(' ', '_'), description


    def run(self):
        file_reader = pyfastaq.sequences.file_reader(self.infile)
        fa_out = pyfastaq.utils.open_file_write(self.outprefix + '.fa')
        tsv_out = pyfastaq.utils.open_file_write(self.outprefix + '.tsv')

        for seq in file_reader:
            original_id = seq.id
            seq.id, description = self._fa_header_to_name_and_metadata(seq.id)
            if description == '.':
                seq.id = original_id.split()[0]
            print(seq.id, '1', '0', '.', '.', 'Original name: ' + original_id, sep='\t', file=tsv_out)
            print(seq, file=fa_out)

        pyfastaq.utils.close(fa_out)
        pyfastaq.utils.close(tsv_out)




import unittest
import os
from ariba import mlst_profile

modules_dir = os.path.dirname(os.path.abspath(mlst_profile.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestMlstProfile(unittest.TestCase):
    def test_init(self):
        '''test init'''
        infile = os.path.join(data_dir, 'mlst_profile_test.init.profile.tsv')
        profile = mlst_profile.MlstProfile(infile)
        expected_genes = ['nusA', 'rpoB', 'eno', 'gltB', 'lepA', 'nuoL', 'nrdA']
        self.assertEqual(expected_genes, profile.genes_list)
        self.assertEqual(set(expected_genes), profile.genes_set)
        expected_dict = {
            (1, 26, 2, 2, 59, 8, 1): 1,
            (1, 26, 2, 4, 59, 2, 5): 2
        }
        self.assertEqual(expected_dict, profile.profile_to_type)
		
    def test_init_multiple_extra_columns(self):
        '''test init'''
        infile = os.path.join(data_dir, 'mlst_profile_test.init_multiple_extra_columns.profile.tsv')
        profile = mlst_profile.MlstProfile(infile)
        expected_genes = ['nusA', 'rpoB', 'eno', 'gltB', 'lepA', 'nuoL', 'nrdA']
        self.assertEqual(expected_genes, profile.genes_list)
        self.assertEqual(set(expected_genes), profile.genes_set)
        expected_dict = {
            (1, 26, 2, 2, 59, 8, 1): 1,
            (1, 26, 2, 4, 59, 2, 5): 2
        }
        self.assertEqual(expected_dict, profile.profile_to_type)


    def test_has_gene(self):
        '''Test has_gene'''
        infile = os.path.join(data_dir, 'mlst_profile_test.profile.tsv')
        profile = mlst_profile.MlstProfile(infile)
        for gene in ['nusA', 'rpoB', 'eno', 'gltB', 'lepA', 'nuoL', 'nrdA']:
            self.assertTrue(profile.has_gene(gene))
        self.assertFalse(profile.has_gene('not there'))


    def test_get_sequence_type(self):
        '''Test get_sequence_type'''
        infile = os.path.join(data_dir, 'mlst_profile_test.profile.tsv')
        profile = mlst_profile.MlstProfile(infile)
        tests = [
            ('ND', {}),
            ('ND', {'foo': 1}),
            (1, {'nusA': 1, 'rpoB': 26, 'eno': 2, 'gltB': 2, 'lepA': 59, 'nuoL': 8, 'nrdA': 1}),
            (2, {'nusA': 1, 'rpoB': 26, 'eno': 2, 'gltB': 4, 'lepA': 59, 'nuoL': 2, 'nrdA': 5}),
            ('ND', {'nusA': 'ND', 'rpoB': 26, 'eno': 2, 'gltB': 4, 'lepA': 59, 'nuoL': 2, 'nrdA': 5}),
            ('Novel', {'nusA': 1000, 'rpoB': 26, 'eno': 2, 'gltB': 4, 'lepA': 59, 'nuoL': 2, 'nrdA': 5}),
        ]
        for expected, profile_dict in tests:
            self.assertEqual(expected, profile.get_sequence_type(profile_dict))




from ariba import flag, report, summary_cluster_variant

class Error (Exception): pass

int_columns = [
    'reads',
    'ref_len',
    'ref_base_assembled',
    'ctg_len',
    'ref_start',
    'ref_end',
    'ctg_start',
    'ctg_end',
]


float_columns = ['pc_ident', 'ctg_cov']


class SummaryCluster:
    def __init__(self, min_pc_id=90):
        self.min_pc_id = min_pc_id
        self.name = None
        self.ref_name = None
        self.flag = None
        self.data = []


    def __eq__(self, other):
       return type(other) is type(self) and self.__dict__ == other.__dict__


    @classmethod
    def line2dict(cls, line, filename=None):
        data = line.rstrip().split('\t')
        if len(data) != len(report.columns):
            if filename is not None:
                filename_message = 'Error reading ariba summary file "' + filename + '". '
            else:
                filename_message = ''
            raise Error(filename_message + 'Wrong number of columns in the following line. Expected ' + str(len(report.columns)) + ' but got ' + str(len(data)) + '\n' + line)

        d = {report.columns[i]: data[i] for i in range(len(data))}
        try:
            d['flag'] = flag.Flag(int(d['flag']) )
        except:
            raise Error('Error getting flag in the following line. Got "' + d['flag'] + '" for the flag.\n' + line)

        for key in int_columns:
            try:
                d[key] = int(d[key])
            except:
                assert d[key] == '.'

        for key in float_columns:
            try:
                d[key] = float(d[key])
            except:
                assert d[key] == '.'

        if d['var_description'] == '.':
            d['var_group'] = '.'
        else:
            try:
                d['var_group'] = d['var_description'].split(':')[4]
            except:
                raise Error('Error getting variant group from the following line:\n' + line)

        return d


    def add_data_dict(self, data_dict):
        if data_dict['pc_ident'] == '.' or data_dict['pc_ident'] < self.min_pc_id:
            return

        if self.name is None:
            assert self.ref_name is None and self.flag is None
            self.name = data_dict['cluster']
            self.ref_name = data_dict['ref_name']
            self.flag = data_dict['flag']

        if self.name != data_dict['cluster']:
            raise Error('Cannot add dict to SummaryCluster. Expected cluster name "' + self.name + '" but got "' + data_dict['cluster'] + '".')

        if self.ref_name != data_dict['ref_name']:
            raise Error('Cannot add dict to SummaryCluster. Expected ref_name "' + self.ref_name + '" but got "' + data_dict['ref_name'] + '".')

        if self.flag != data_dict['flag']:
            raise Error('Cannot add dict to SummaryCluster. Expected flag "' + str(self.flag) + '" but got "' + str(data_dict['flag']) + '".')
        self.data.append(data_dict)


    def _pc_id_and_read_depth_of_longest(self):
        longest = 0
        identity = 0
        depth = 0

        for d in self.data:
            if d['ref_base_assembled'] > longest:
                longest = d['ref_base_assembled']
                identity = d['pc_ident']
                depth = d['ctg_cov']

        return identity, depth


    def _has_any_part_of_ref_assembled(self):
        for d in self.data:
            if isinstance(d['ref_base_assembled'], int) and d['ref_base_assembled'] > 0:
                return True

        return False


    def _to_cluster_summary_assembled(self):
        if len(self.data) == 0:
            return 'no'

        if self.data[0]['gene'] == '0':
            has_complete_gene = True
        else:
            has_complete_gene = self.flag.has('complete_gene')

        if self.flag.has('assembly_fail') or self.flag.has('ref_seq_choose_fail'):
            return 'no'
        elif not self.flag.has('assembled'):
            return 'partial' if self._has_any_part_of_ref_assembled() else 'no'
        elif self.flag.has('assembled_into_one_contig') and has_complete_gene:
            if self.flag.has('unique_contig') and \
              (not self.flag.has('scaffold_graph_bad')) and \
              (not self.flag.has('variants_suggest_collapsed_repeat')) and \
              (not self.flag.has('hit_both_strands')) and \
              (not self.flag.has('region_assembled_twice')):
                return 'yes'
            else:
                return 'yes_nonunique'
        elif self.flag.has('assembled_into_one_contig'):
            return 'interrupted'
        else:
            return 'fragmented'


    @classmethod
    def _has_known_variant(cls, data_dict):
        if data_dict['has_known_var'] == '1':
            return 'yes'
        elif data_dict['known_var'] == '0':
            return 'no'
        elif data_dict['gene'] == '1': # we don't yet call hets in genes
            return 'no'
        else:
            cluster_var = summary_cluster_variant.SummaryClusterVariant(data_dict)
            return 'het' if cluster_var.is_het else 'no'


    def _has_any_known_variant(self):
        results = {self._has_known_variant(d) for d in self.data}
        if 'yes' in results:
            return 'yes'
        else:
            return 'het' if 'het' in results else 'no'


    @classmethod
    def _has_nonsynonymous(cls, data_dict):
        cluster_var = summary_cluster_variant.SummaryClusterVariant(data_dict)

        has_non_het = data_dict['ref_ctg_effect'] != 'SYN' and \
        (
            data_dict['has_known_var'] == '1' or \
            (data_dict['known_var'] != '1' and (data_dict['ref_ctg_change'] != '.' or data_dict['ref_ctg_effect'] != '.'))
        )

        if has_non_het and not cluster_var.is_het:
            return 'yes'
        else:
            return 'het' if cluster_var.is_het else 'no'


    def _has_any_nonsynonymous(self):
        results = {SummaryCluster._has_nonsynonymous(d) for d in self.data}

        if 'yes' in results:
            return 'yes'
        else:
            return 'het' if 'het' in results else 'no'


    @classmethod
    def _has_novel_nonsynonymous(cls, data_dict):
        has_nonsynon = SummaryCluster._has_nonsynonymous(data_dict)
        if has_nonsynon == 'no':
            return 'no'
        else:
            has_known = SummaryCluster._has_known_variant(data_dict)
            if has_known == 'no':
                return has_nonsynon
            else:
                return 'no'


    def _has_any_novel_nonsynonymous(self):
        results = {SummaryCluster._has_novel_nonsynonymous(d) for d in self.data}

        if 'yes' in results:
            return 'yes'
        else:
            return 'het' if 'het' in results else 'no'


    def _to_cluster_summary_has_known_nonsynonymous(self, assembled_summary):
        '''assembled_summary should be output of _to_cluster_summary_assembled'''
        if assembled_summary == 'no':
            return 'NA'
        else:
            return self._has_any_known_variant()


    def _to_cluster_summary_has_novel_nonsynonymous(self, assembled_summary):
        '''assembled_summary should be output of _to_cluster_summary_assembled'''
        if assembled_summary == 'no':
            return 'NA'
        else:
            return self._has_any_novel_nonsynonymous()


    def _to_cluster_summary_has_nonsynonymous(self, assembled_summary):
        '''assembled_summary should be output of _to_cluster_summary_assembled'''
        if assembled_summary == 'no':
            return 'NA'
        else:
            return self._has_any_nonsynonymous()


    @staticmethod
    def _get_known_noncoding_het_snp(data_dict):
        '''If ref is coding, return None. If the data dict has a known snp, and
           samtools made a call, then return the string ref_name_change and the
           % of reads supporting the variant type. If noncoding, but no
           samtools call, then return None'''
        if data_dict['gene'] == '1':
            return None

        if data_dict['known_var'] == '1' and data_dict['ref_ctg_effect'] == 'SNP' \
          and data_dict['smtls_nts'] != '.' and ';' not in data_dict['smtls_nts']:
            nucleotides = data_dict['smtls_nts'].split(',')
            depths = data_dict['smtls_nts_depth'].split(',')

            if len(nucleotides) != len(depths):
                raise Error('Mismatch in number of inferred nucleotides from ctg_nt, smtls_nts, smtls_nts_depth columns. Cannot continue\n' + str(data_dict))

            try:
                var_nucleotide = data_dict['known_var_change'][-1]
                depths = [int(x) for x in depths]
                nuc_to_depth = dict(zip(nucleotides, depths))
                total_depth = sum(depths)
                var_depth = nuc_to_depth.get(var_nucleotide, 0)
                percent_depth = round(100 * var_depth / total_depth, 1)
            except:
                return None

            return data_dict['known_var_change'], percent_depth
        else:
            return None


    @staticmethod
    def _get_nonsynonymous_var(data_dict):
        '''if data_dict has a non synonymous variant, return string:
        ref_name.change. Otherwise return None'''
        has_nonsyn = SummaryCluster._has_nonsynonymous(data_dict)

        if has_nonsyn == 'no':
            return None
        elif data_dict['known_var_change'] == data_dict['ref_ctg_change'] == '.' == data_dict['ref_ctg_effect']:
            raise Error('Unexpected data in ariba summary... \n' + str(data_dict) + '\n... known_var_change, ref_ctg_change, ref_ctg_effect all equal to ".", but has a non synonymous change. Something is inconsistent. Cannot continue')
        else:
            if '.' not in [data_dict['known_var_change'], data_dict['ref_ctg_change']] and \
              data_dict['known_var_change'] != data_dict['ref_ctg_change']:
                raise Error('Unexpected data in ariba summary... \n' + str(data_dict) + '\n... known_var_change != ref_ctg_change. Cannot continue')

            var_group = 'novel', None

            if data_dict['known_var'] == '1' and data_dict['known_var_change'] != '.':
                var_change = data_dict['known_var_change']
                if data_dict['var_group'] == '.':
                    var_group = 'ungrouped', None
                else:
                    var_group = 'grouped', data_dict['var_group']
            elif data_dict['ref_ctg_change'] != '.':
                var_change = data_dict['ref_ctg_change']
            else:
                var_change = data_dict['ref_ctg_effect']

            return (data_dict['ref_name'], var_change) + var_group


    def _has_match(self, assembled_summary):
        '''assembled_summary should be output of _to_cluster_summary_assembled'''
        if assembled_summary.startswith('yes'):
            if self.data[0]['var_only'] == '0' or self._to_cluster_summary_has_known_nonsynonymous(assembled_summary) == 'yes':
                return 'yes'
            else:
                return 'no'
        else:
            return 'no'


    def has_var_groups(self):
        '''Returns a set of the variant group ids that this cluster has'''
        ids = set()
        for d in self.data:
            if self._has_known_variant(d) != 'no' and d['var_group'] != '.':
                ids.add(d['var_group'])
        return ids


    def column_summary_data(self):
        '''Returns a dictionary of column name -> value, for cluster-level results'''
        assembled_summary = self._to_cluster_summary_assembled()
        pct_id, read_depth = self._pc_id_and_read_depth_of_longest()

        columns = {
            'assembled': self._to_cluster_summary_assembled(),
            'match': self._has_match(assembled_summary),
            'ref_seq': self.ref_name,
            'pct_id': str(pct_id),
            'ctg_cov': str(read_depth),
            'known_var': self._to_cluster_summary_has_known_nonsynonymous(assembled_summary),
            'novel_var': self._to_cluster_summary_has_novel_nonsynonymous(assembled_summary)
        }

        return columns


    def non_synon_variants(self):
        variants = {self._get_nonsynonymous_var(d) for d in self.data}
        variants.discard(None)
        return variants


    def known_noncoding_het_snps(self):
        snps = {}
        for d in self.data:
            snp_tuple = self._get_known_noncoding_het_snp(d)
            if snp_tuple is not None:
                try:
                    snp_id = d['var_description'].split(':')[4]
                except:
                    raise Error('Error getting ID from ' + str(d) + '\n' + d['var_description'])
                if snp_id not in snps:
                    snps[snp_id] = {}
                snps[snp_id][snp_tuple[0]] = snp_tuple[1]
        return snps


    @classmethod
    def _get_all_nonsynon_variants_set(cls, data_dicts):
        variants = set()

        for data_dict in data_dicts:
            cluster_var = summary_cluster_variant.SummaryClusterVariant(data_dict)
            if cluster_var.has_nonsynon or cluster_var.is_het:
                variants.add(cluster_var)

        return variants


    def gather_data(self):
        self.summary = self.column_summary_data()
        self.variants = self._get_all_nonsynon_variants_set(self.data)




import filecmp
import json
import os
import unittest

import pyfastaq

from ariba import common, tb

modules_dir = os.path.dirname(os.path.abspath(tb.__file__))
data_dir = os.path.join(modules_dir, 'tests', 'data')


class TestTb(unittest.TestCase):
    def test_report_to_resistance_dict(self):
        '''test report_to_resistance_dict'''
        infile = os.path.join(data_dir, 'tb_report_to_resistance_dict.tsv')
        got = tb.report_to_resistance_dict(infile)
        expect = {
            'Ethambutol': [('ref1', 'I42J'), ('ref2_upstream', 'A-3G')],
            'Rifampicin': [('ref1', 'I42J')],
            'Isoniazid': [('katG', 'Incomplete_gene')],
        }
        self.assertEqual(expect, got)


    def test_genbank_to_gene_coords(self):
        '''test genbank_to_gene_coords'''
        infile = os.path.join(data_dir, 'tb_genbank_to_gene_coords.gb')
        expect = {
            'gene1': {'start': 42, 'end': 45},
            'gene2': {'start': 10, 'end': 5},
        }
        genes = {'gene1', 'gene2'}

        got = tb.genbank_to_gene_coords(infile, genes)
        self.assertEqual(expect, got)



    def test_load_mutations(self):
        '''test load_mutations'''
        mutation_to_drug_json = 'tmp.tb.load_mutations.json'
        variants_txt = 'tmp.tb.load_mutations.txt'

        gene_coords = {
            'gene1': {'start': 42, 'end': 45},
            'gene2': {'start': 100, 'end': 50},
            'gene3': {'start': 200, 'end': 300},
        }

        mutation_to_drug = {
            'gene1_A-5T': ['drug1'],
            'gene1_A5C': ['drug1', 'drug2'],
            'gene2_A-3G': ['drug4'],
            'gene2_A10C': ['drug5'],
            'gene3_AC10A': ['drug5'],
        }
        with open(mutation_to_drug_json, 'w') as f:
            json.dump(mutation_to_drug, f, indent=4, sort_keys=True)

        with open(variants_txt, 'w') as f:
            print('gene1', 'A-5T', 'DNA', sep='\t', file=f)
            print('gene1', 'A5C', 'PROT', sep='\t', file=f)
            print('gene2', 'A-3G', 'DNA', sep='\t', file=f)
            print('gene2', 'A10C', 'PROT', sep='\t', file=f)
            print('gene3', 'AC10A', 'DNA', sep='\t', file=f)
        got_mutations, got_genes_with_indels, got_genes_need_upstream, got_genes_non_upstream = tb.load_mutations(gene_coords, mutation_to_drug_json, variants_txt, upstream_before=10)
        expect_mutations = [
            {'gene': 'gene1', 'var': 'A6T', 'coding': 1, 'upstream': True, 'drugs': 'drug1', 'original_mutation': 'A-5T'},
            {'gene': 'gene1', 'var': 'A5C', 'coding': 1, 'upstream': False, 'drugs': 'drug1,drug2'},
            {'gene': 'gene2', 'var': 'A8G', 'coding': 1, 'upstream': True, 'drugs': 'drug4', 'original_mutation': 'A-3G'},
            {'gene': 'gene2', 'var': 'A10C', 'coding': 1, 'upstream': False, 'drugs': 'drug5'}
        ]
        expect_genes_with_indels = {'gene3'}
        expect_genes_need_upstream = {'gene1', 'gene2'}
        expect_genes_non_upstream = {'gene1', 'gene2'}
        self.assertEqual(expect_mutations, got_mutations)
        self.assertEqual(expect_genes_with_indels, got_genes_with_indels)
        self.assertEqual(expect_genes_need_upstream, got_genes_need_upstream)
        self.assertEqual(expect_genes_non_upstream, got_genes_non_upstream)
        os.unlink(mutation_to_drug_json)
        os.unlink(variants_txt)


    def test_write_prepareref_fasta_file(self):
        '''test write_prepareref_fasta_file'''
        outfile = 'tmp.write_prepareref_fasta_file.fa'
        # start of the ref seq is:
        # 0         10        20
        # 0123456789012345678901234567890
        # TTGACCGATGACCCCGGTTCAGGCTTCACCA
        gene_coords = {
            'gene1': {'start': 0, 'end': 6},
            'gene2': {'start': 12, 'end': 10},
            'gene3': {'start': 13, 'end': 16},

        }
        expect_seqs = {
            'gene1': pyfastaq.sequences.Fasta('gene1', 'TTGACCG'),
            'gene2': pyfastaq.sequences.Fasta('gene2', 'GGT'),
            'gene2_upstream': pyfastaq.sequences.Fasta('gene2_upstream', 'CGGGGTCA'),
            'gene3_upstream': pyfastaq.sequences.Fasta('gene3_upstream', 'ACCCCGGT'),
        }
        genes_need_upstream = {'gene2', 'gene3'}
        genes_non_upstream = {'gene1', 'gene2'}
        tb.write_prepareref_fasta_file(outfile, gene_coords, genes_need_upstream, genes_non_upstream, upstream_before=3, upstream_after=5)
        got_seqs = {}
        pyfastaq.tasks.file_to_dict(outfile, got_seqs)
        os.unlink(outfile)
        self.assertEqual(expect_seqs, got_seqs)


    def test_write_prepareref_metadata_file(self):
        '''test write_prepareref_metadata_file'''
        mutations = [
            {'gene': 'gene1', 'var': 'A6T', 'coding': 1, 'upstream': True, 'drugs': 'drug1', 'original_mutation': 'A-5T'},
            {'gene': 'gene1', 'var': 'A5C', 'coding': 1, 'upstream': False, 'drugs': 'drug1,drug2'},
            {'gene': 'gene2', 'var': 'A8G', 'coding': 1, 'upstream': True, 'drugs': 'drug4', 'original_mutation': 'A-3G'},
            {'gene': 'gene2', 'var': 'A10C', 'coding': 1, 'upstream': False, 'drugs': 'drug5'}
        ]
        outfile = 'tmp.write_prepareref_metadata_file.tsv'
        tb.write_prepareref_metadata_file(mutations, outfile)
        expected = os.path.join(data_dir, 'tb_write_prepareref_metadata_file.tsv')
        self.assertTrue(filecmp.cmp(expected, outfile, shallow=False))
        os.unlink(outfile)


    def test_make_prepareref_files(self):
        '''test make_prepareref_files'''
        outprefix = 'tmp.make_prepareref_files'
        expected_files = [outprefix + '.fa', outprefix + '.tsv']
        for fname in expected_files:
            if os.path.exists(fname):
                os.unlink(fname)
        tb.make_prepareref_files(outprefix)

        for fname in expected_files:
            self.assertTrue(os.path.exists(fname))
            os.unlink(fname)


    def test_make_prepareref_dir(self):
        '''test make_prepareref_dir'''
        outdir = 'tmp.make_prepareref_dir'
        common.rmtree(outdir)
        tb.make_prepareref_dir(outdir)
        self.assertTrue(os.path.exists(outdir))
        json_file = os.path.join(outdir, '00.params.json')
        common.rmtree(outdir)

